@@ -5,28 +5,54 @@
  *  Created   : 1991-04-01
  *  Updated   : 1998-07-13
  *  Notes     :
- *  Copyright : (c) Copyright 1991-99 by Iain Lea
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1997-2001 Iain Lea <iain@bricbrac.de>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Iain Lea.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #ifndef EXTERN_H
-#define EXTERN_H 1
+#	define EXTERN_H 1
 
 /*
  * Library prototypes
  */
 
-#ifdef USE_TERMINFO
-#define tigetnum(s) tgetnum(s)
-#endif
+#ifndef RFC2046_H
+#	include <rfc2046.h>
+#endif /* !RFC2046_H */
 
 /*
  * The prototypes bracketed by DECL_xxxx ifdef's are used to get moderately
- * clean compiles on systems with pre-ANSI/POSIX headers when compiler warnings
- * are enabled.  (Not all of the functions are ANSI or POSIX).
+ * clean compiles on systems with pre-ANSI/POSIX headers when compiler
+ * warnings are enabled. (Not all of the functions are ANSI or POSIX).
  */
 #ifdef DECL__FLSBUF
 	extern int _flsbuf (int, FILE *);
@@ -131,7 +157,7 @@
 #endif /* 0 */
 
 #ifdef DECL_MKSTEMP
-	extern int mktemp (char *);
+	extern int mkstemp (char *);
 #endif /* DECL_MKSTEMP */
 #ifdef DECL_MKTEMP
 	extern char *mktemp (char *);
@@ -166,37 +192,17 @@
 #ifdef DECL_SELECT
 	extern int select (int, fd_set *, fd_set *, fd_set *, struct timeval *);
 #endif /* DECL_SELECT */
-#ifdef DECL_SETEGID
-	extern int setegid (gid_t);
-#endif /* DECL_SETEGID */
 #ifndef HAVE_PUTENV
 #	ifdef DECL_SETENV
 		extern int setenv (const char *, const char *, int);
 #	endif /* DECL_SETENV */
 #endif /* !HAVE_PUTENV */
-#ifdef DECL_SETEUID
-	extern int seteuid (uid_t);
-#endif /* DECL_SETEUID */
-#ifdef DECL_SETPGRP
-#	ifdef SETPGRP_VOID
-		extern pid_t setpgrp (void);
-#	else
-		extern int setpgrp (int, int);
-#	endif /* SETPGRP_VOID */
-#endif /* DECL_SETPGRP */
-#ifndef HAVE_SETEGID
-#	ifdef DECL_SETREGID
-		extern int setregid (gid_t, gid_t);
-#	endif /* DECL_SETREGID */
-#endif /* !HAVE_SETEGID */
-#ifndef HAVE_SETEUID
-#	ifdef DECL_SETREUID
-		extern int setreuid (uid_t, uid_t);
-#	endif /* DECL_SETREUID */
-#endif /* !HAVE_SETEUID */
 #ifdef DECL_SOCKET
 	extern int socket (int, int, int);
 #endif /* DECL_SOCKET */
+#ifdef DECL_SNPRINTF
+	extern int snprintf (char *, size_t, const char *, ...);
+#endif /* DECL_SNPRINTF */
 #ifdef DECL_SSCANF
 	extern int sscanf (const char *, const char *, ...);
 #endif /* DECL_SSCANF */
@@ -228,7 +234,7 @@
 #ifdef DECL_TGETFLAG
 	extern int tgetflag (char *);
 #endif /* DECL_TGETFLAG */
-#if defined(DECL_TGETNUM) || defined(DECL_TIGETNUM)
+#if defined(DECL_TGETNUM)
 	extern int tgetnum (char *);
 #endif /* DECL_TGETNUM */
 #ifdef DECL_TGETSTR
@@ -237,6 +243,15 @@
 #ifdef DECL_TGOTO
 	extern char *tgoto (char *, int, int);
 #endif /* DECL_TGOTO */
+#ifdef DECL_TIGETFLAG
+	extern int tigetflag (char *);
+#endif /* DECL_TIGETFLAG */
+#if defined(DECL_TIGETNUM)
+	extern int tigetnum (char *);
+#endif /* DECL_TGETNUM */
+#ifdef DECL_TIGETSTR
+	extern char *tigetstr (char *, char **);
+#endif /* DECL_TIGETSTR */
 #ifdef DECL_TIME
 	extern time_t time (time_t *);
 #endif /* DECL_TIME */
@@ -246,6 +261,9 @@
 #if defined(DECL_TOUPPER) && !defined(toupper)
 	extern int toupper (int);
 #endif /* DECL_TOUPPER && !toupper */
+#ifdef DECL_TPARM
+	extern char *tparm (const char *, ...);
+#endif /* DECL_TPARM */
 #ifdef DECL_TPUTS
 	extern int tputs (char *, int, OutcPtr);
 #endif /* DECL_TPUTS */
@@ -255,28 +273,52 @@
 #ifdef DECL_USLEEP
 	extern void usleep(unsigned long);
 #endif /* DECL_USLEEP */
+#ifdef DECL_VSNPRINTF
+	extern int vsnprintf (char *, size_t, const char *, va_list);
+#endif /* DECL_VSNPRINTF */
 #ifdef DECL_VSPRINTF
 	extern int vsprintf(char *, char *, va_list);
 #endif /* DECL_VSPRINTF */
 
+
 extern int optind;
 extern char *optarg;
 
 /*
  * Local variables
  */
-extern FILE *note_fp;				/* body of current article */
+
+/* This fixes ambiguities on platforms that don't distinguish extern case */
+#ifdef CASE_PROBLEM
+#	define txt_help_B	txt_help_BIGB
+#	define txt_help_D	txt_help_BIGD
+#	define txt_help_I	txt_help_BIGI
+#	define txt_help_K	txt_help_BIGK
+#	define txt_help_L	txt_help_BIGL
+#	define txt_help_M	txt_help_BIGM
+#	define txt_help_Q	txt_help_BIGQ
+#	define txt_help_S	txt_help_BIGS
+#	define txt_help_T	txt_help_BIGT
+#	define txt_help_U	txt_help_BIGU
+#	define txt_help_W	txt_help_BIGW
+#	define txt_help_X	txt_help_BIGX
+#	define txt_help_p_S	txt_help_p_BIGS
+#	define txt_help_t_K	txt_help_t_BIGK
+#	define txt_help_p_U	txt_help_p_BIGU
+#	define txt_help_thd_C	txt_help_thd_BIGC
+#endif /* CASE_PROBLEM */
+
 extern char **news_headers_to_display_array;
 extern char **news_headers_to_not_display_array;
 extern char *OPT_CHAR_list[];
 extern char *OPT_STRING_list[];
+extern char *ch_post_process;
 extern char *glob_group;
 extern char *nntp_server;
 extern char active_times_file[PATH_LEN];
 extern char bug_addr[LEN];
 extern char bug_nntpserver1[PATH_LEN];
 extern char bug_nntpserver2[PATH_LEN];
-extern char ch_post_process[];
 extern char cmdline_nntpserver[PATH_LEN];
 extern char cvers[LEN];
 extern char dead_article[PATH_LEN];
@@ -305,8 +347,6 @@ extern char mailbox[PATH_LEN];
 extern char mailer[PATH_LEN];
 extern char mailgroups_file[PATH_LEN];
 extern char mesg[LEN];
-extern char msg_headers_file[PATH_LEN];
-extern char my_distribution[LEN];
 extern char newnewsrc[PATH_LEN];
 extern char news_active_file[PATH_LEN];
 extern char newsgroups_file[PATH_LEN];
@@ -320,7 +360,6 @@ extern char postponed_articles_file[PATH_LEN];
 extern char proc_ch_default;
 extern char tin_progname[PATH_LEN];
 extern char rcdir[PATH_LEN];
-extern char reply_to[LEN];
 extern char save_active_file[PATH_LEN];
 extern char spooldir[PATH_LEN];
 extern char subscriptions_file[PATH_LEN];
@@ -331,7 +370,10 @@ extern char domain_name[];
 extern char host_name[];
 
 extern const char base64_alphabet[64];
+
 extern const char **info_help;
+extern constext *content_encodings[];
+extern constext *content_types[];
 extern constext *help_group[];
 extern constext *help_page[];
 extern constext *help_select[];
@@ -352,6 +394,7 @@ extern constext txt_7bit[];
 extern constext txt_8bit[];
 extern constext txt_active_file_is_empty[];
 extern constext txt_added_groups[];
+extern constext txt_all[];
 extern constext txt_all_groups[];
 extern constext txt_append_overwrite_quit[];
 extern constext txt_art_cancel[];
@@ -365,13 +408,18 @@ extern constext txt_art_parent_none[];
 extern constext txt_art_parent_unavail[];
 extern constext txt_art_posted[];
 extern constext txt_art_rejected[];
-extern constext txt_art_saved_to[];
 extern constext txt_art_thread_regex_tag[];
 extern constext txt_art_unavailable[];
 extern constext txt_article_cancelled[];
-extern constext txt_article_cannot_open[];
-extern constext txt_article_cannot_reopen[];
+extern constext txt_article_plural[];
 extern constext txt_article_reposted[];
+extern constext txt_article_singular[];
+extern constext txt_articles_mailed[];
+#ifndef DISABLE_PRINTING
+	extern constext txt_articles_printed[];
+#endif /* !DISABLE_PRINTING */
+extern constext txt_attach[];
+extern constext txt_uue[];
 extern constext txt_at_s[];
 extern constext txt_auth_failed[];
 extern constext txt_auth_pass[];
@@ -391,21 +439,17 @@ extern constext txt_base64[];
 extern constext txt_batch_update_unavail[];
 extern constext txt_batch_update_failed[];
 extern constext txt_begin_of_art[];
-extern constext txt_caching_disabled[];
 extern constext txt_cancel_article[];
 extern constext txt_cancelling_art[];
 extern constext txt_cannot_create[];
 extern constext txt_cannot_create_uniq_name[];
 extern constext txt_cannot_find_base_art[];
 extern constext txt_cannot_get_nntp_server_name[];
-extern constext txt_cannot_get_term[];
-extern constext txt_cannot_get_term_entry[];
+#ifndef USE_CURSES
+	extern constext txt_cannot_get_term_entry[];
+#endif /* !USE_CURSES */
 extern constext txt_cannot_open[];
-
-#if defined(NNTP_ABLE) || defined(NNTP_ONLY)
-	extern constext txt_cannot_open_active_file[];
-#endif /* NNTP_ABLE || NNTP_ONLY */
-
+extern constext txt_cannot_open_for_saving[];
 extern constext txt_cannot_post[];
 extern constext txt_cannot_post_group[];
 extern constext txt_cannot_write_index[];
@@ -413,23 +457,26 @@ extern constext txt_cannot_write_to_directory[];
 extern constext txt_catchup_all_read_groups[];
 extern constext txt_catchup_despite_tags[];
 extern constext txt_catchup_update_info[];
+extern constext txt_caughtup[];
 extern constext txt_check_article[];
 extern constext txt_checking_for_news[];
 extern constext txt_checking_new_groups[];
-extern constext txt_checksum_of_file[];
+#ifndef HAVE_LIBUU
+	extern constext txt_checksum_of_file[];
+#endif /* !HAVE_LIBUU */
 extern constext txt_choose_post_process_type[];
 extern constext txt_color_off[];
 extern constext txt_color_on[];
 extern constext txt_command_failed[];
 extern constext txt_connecting[];
 extern constext txt_connecting_port[];
-extern constext txt_continue[];
 extern constext txt_copyright_notice[];
 extern constext txt_creating_active[];
 extern constext txt_creating_newsrc[];
 extern constext txt_default[];
 extern constext txt_delete_processed_files[];
 extern constext txt_deleting[];
+extern constext txt_end_of_art[];
 extern constext txt_end_of_arts[];
 extern constext txt_end_of_groups[];
 extern constext txt_end_of_thread[];
@@ -491,10 +538,14 @@ extern constext txt_error_no_from[];
 extern constext txt_error_no_read_permission[];
 extern constext txt_error_no_such_file[];
 extern constext txt_error_no_write_permission[];
+extern constext txt_error_plural[];
 extern constext txt_error_sender_in_header_not_allowed[];
+extern constext txt_error_singular[];
 extern constext txt_exiting[];
 extern constext txt_external_mail_done[];
-extern constext txt_extracting_archive[];
+#ifndef HAVE_LIBUU
+	extern constext txt_extracting_archive[];
+#endif /* !HAVE_LIBUU */
 extern constext txt_extracting_shar[];
 extern constext txt_failed_to_connect_to_server[];
 extern constext txt_feed_pattern[];
@@ -510,10 +561,13 @@ extern constext txt_followup_poster[];
 extern constext txt_from_line_only[];
 extern constext txt_from_line_only_case[];
 extern constext txt_full[];
+extern constext txt_global[];
 extern constext txt_group[];
 extern constext txt_group_is_moderated[];
+extern constext txt_group_plural[];
 extern constext txt_group_select_com[];
 extern constext txt_group_selection[];
+extern constext txt_group_singular[];
 extern constext txt_help_disp[];
 extern constext txt_help_disp_[];
 extern constext txt_help_navi[];
@@ -523,6 +577,7 @@ extern constext txt_help_misc_[];
 extern constext txt_help_ops[];
 extern constext txt_help_ops_[];
 extern constext txt_help_B[];
+extern constext txt_help_B_[];
 extern constext txt_help_D[];
 extern constext txt_help_I[];
 extern constext txt_help_K[];
@@ -572,6 +627,7 @@ extern constext txt_help_g_num[];
 extern constext txt_help_g_q[];
 extern constext txt_help_g_r[];
 extern constext txt_help_g_search[];
+extern constext txt_help_g_search_[];
 extern constext txt_help_g_T[];
 extern constext txt_help_g_tab[];
 extern constext txt_help_g_x[];
@@ -596,12 +652,9 @@ extern constext txt_help_kill_scope[];
 extern constext txt_help_l[];
 extern constext txt_help_m[];
 extern constext txt_help_n[];
-
-#ifndef DISABLE_PRINTING
-	extern constext txt_help_o[];
-#endif /* !DISABLE_PRINTING */
-
 extern constext txt_help_p_S[];
+extern constext txt_help_p_U[];
+extern constext txt_help_p_V[];
 extern constext txt_help_p_caret_dollar[];
 extern constext txt_help_p_at[];
 extern constext txt_help_p_cr[];
@@ -612,6 +665,8 @@ extern constext txt_help_p_dot[];
 extern constext txt_help_p_f[];
 extern constext txt_help_p_g[];
 extern constext txt_help_p_k[];
+extern constext txt_help_p_lb[];
+extern constext txt_help_p_rb[];
 extern constext txt_help_p_m[];
 extern constext txt_help_p_n[];
 extern constext txt_help_p_num[];
@@ -624,20 +679,12 @@ extern constext txt_help_p_tab[];
 extern constext txt_help_p_tilda[];
 extern constext txt_help_p_u[];
 extern constext txt_help_p_z[];
-
-#ifndef DONT_HAVE_PIPING
-	extern constext txt_help_pipe[];
-#endif /* !DONT_HAVE_PIPING */
-
 extern constext txt_help_plus[];
-
 extern constext txt_help_q[];
 extern constext txt_help_quick_kill[];
 extern constext txt_help_quick_select[];
-
 extern constext txt_help_r[];
 extern constext txt_help_s[];
-extern constext txt_help_s_i[];
 extern constext txt_help_sel_c[];
 extern constext txt_help_select_scope[];
 extern constext txt_help_semicolon[];
@@ -669,7 +716,15 @@ extern constext txt_invalid_from[];
 extern constext txt_invalid_sender[];
 extern constext txt_inverse_off[];
 extern constext txt_inverse_on[];
-extern constext txt_is_tex_ecoded[];
+extern constext txt_is_mailbox[];
+extern constext txt_is_tex_encoded[];
+extern constext txt_keymap_missing_key[];
+extern constext txt_keymap_invalid_key[];
+extern constext txt_keymap_invalid_name[];
+#ifdef DEBUG
+	extern constext txt_keymap_redef[];
+#endif /* DEBUG */
+extern constext txt_keymap_conflict[];
 extern constext txt_kill_from[];
 extern constext txt_kill_lines[];
 extern constext txt_kill_menu[];
@@ -680,22 +735,10 @@ extern constext txt_kill_text[];
 extern constext txt_kill_time[];
 extern constext txt_last[];
 extern constext txt_last_resp[];
-
-#ifdef HAVE_LIBUU
-	extern constext txt_libuu_saved[];
-	extern constext txt_libuu_success[];
-	extern constext txt_libuu_error_decode[];
-	extern constext txt_libuu_error_missing[];
-	extern constext txt_libuu_error_no_begin[];
-	extern constext txt_libuu_error_no_end[];
-	extern constext txt_libuu_error_no_data[];
-	extern constext txt_libuu_error_unknown[];
-#endif /* HAVE_LIBUU */
-
 extern constext txt_lines[];
-extern constext txt_listing_archive[];
 extern constext txt_mail[];
 extern constext txt_mail_art_to[];
+extern constext txt_mail_log_to[];
 extern constext txt_mail_bug_report[];
 extern constext txt_mail_bug_report_confirm[];
 extern constext txt_mailed[];
@@ -718,26 +761,19 @@ extern constext txt_mini_thread_1[];
 extern constext txt_mini_thread_2[];
 extern constext txt_more[];
 extern constext txt_moving[];
-extern constext txt_msg_headers_file[];
 extern constext txt_msgid_line_only[];
 extern constext txt_newsgroup[];
+extern constext txt_newsgroup_plural[];
 extern constext txt_newsgroup_position[];
+extern constext txt_newsgroup_singular[];
 extern constext txt_newsrc_again[];
 extern constext txt_newsrc_nogroups[];
 extern constext txt_newsrc_saved[];
 extern constext txt_next_resp[];
 extern constext txt_nntp_authorization_failed[];
-#ifdef NNTP_ABLE
-	extern constext txt_nntp_ok_goodbye[];
-#endif /* NNTP_ABLE */
 extern constext txt_no[];
 extern constext txt_no_arts[];
 extern constext txt_no_arts_posted[];
-
-#ifndef DONT_HAVE_PIPING
-	extern constext txt_no_command[];
-#endif /* !DONT_HAVE_PIPING */
-
 extern constext txt_no_description[];
 extern constext txt_no_filename[];
 extern constext txt_no_group[];
@@ -763,6 +799,7 @@ extern constext txt_no_term_clear_eos[];
 extern constext txt_no_term_clearscreen[];
 extern constext txt_no_term_cursor_motion[];
 extern constext txt_no_term_set[];
+extern constext txt_no_viewer_found[];
 extern constext txt_no_xover_support[];
 extern constext txt_not_exist[];
 extern constext txt_not_in_active_file[];
@@ -770,35 +807,27 @@ extern constext txt_nrctbl_create[];
 extern constext txt_nrctbl_default[];
 extern constext txt_nrctbl_info[];
 extern constext txt_only[];
-
 extern constext txt_option_not_enabled[];
 extern constext txt_options_menu[];
 extern constext txt_out_of_memory[];
 extern constext txt_pcre_error_at[];
 extern constext txt_pcre_error_num[];
 extern constext txt_pcre_error_text[];
-extern constext txt_plural[];
 extern constext txt_post_a_followup[];
 extern constext txt_post_error_ask_postpone[];
 extern constext txt_post_history_menu[];
 extern constext txt_post_newsgroups[];
 extern constext txt_post_processing[];
-extern constext txt_post_processing_failed[];
 extern constext txt_post_processing_finished[];
 extern constext txt_post_subject[];
+extern constext txt_post_via_builtin_inews[];
+extern constext txt_post_via_builtin_inews_only[];
 extern constext txt_posted_info_file[];
 extern constext txt_posting[];
 extern constext txt_postpone_repost[];
-
-#ifndef DISABLE_PRINTING
-	extern constext txt_print[];
-	extern constext txt_printed[];
-	extern constext txt_printing[];
-#endif /* !DISABLE_PRINTING */
-
 extern constext txt_prompt_fup_ignore[];
 extern constext txt_prompt_unchanged_art[];
-extern constext txt_prompt_unchanged_bug[];
+extern constext txt_prompt_unchanged_mail[];
 extern constext txt_prompt_see_postponed[];
 extern constext txt_quick_filter_kill[];
 extern constext txt_quick_filter_select[];
@@ -824,6 +853,7 @@ extern constext txt_reading_config_file[];
 extern constext txt_reading_filter_file[];
 extern constext txt_reading_groups[];
 extern constext txt_reading_input_history_file[];
+extern constext txt_reading_keymap_file[];
 extern constext txt_reading_news_active_file[];
 extern constext txt_reading_news_newsrc_file[];
 extern constext txt_reading_newsgroups_file[];
@@ -841,13 +871,14 @@ extern constext txt_resp_to_poster[];
 extern constext txt_resp_x_of_n[];
 extern constext txt_return_key[];
 extern constext txt_save[];
+extern constext txt_save_attachment[];
 extern constext txt_save_config[];
 extern constext txt_save_filename[];
 extern constext txt_saved[];
-extern constext txt_saved_arts[];
+extern constext txt_saved_group[];
+extern constext txt_saved_groupname[];
 extern constext txt_saved_nothing[];
-extern constext txt_saved_pattern_to[];
-extern constext txt_saved_to_mailbox[];
+extern constext txt_saved_summary[];
 extern constext txt_saving[];
 extern constext txt_screen_init_failed[];
 extern constext txt_screen_too_small[];
@@ -872,12 +903,9 @@ extern constext txt_select_thread[];
 extern constext txt_select_time[];
 extern constext txt_server_name_in_file_env_var[];
 extern constext txt_servers_active[];
-
-#ifndef NO_SHELL_ESCAPE
-	extern constext txt_shell_escape[];
-#endif /* !NO_SHELL_ESCAPE */
-
 extern constext txt_skipping_newgroups[];
+extern constext txt_space[];
+extern constext txt_starting_command[];
 extern constext txt_subj_line_only[];
 extern constext txt_subj_line_only_case[];
 extern constext txt_subscribe_pattern[];
@@ -888,32 +916,27 @@ extern constext txt_supersede_article[];
 extern constext txt_supersede_group[];
 extern constext txt_superseding_art[];
 extern constext txt_suspended_message[];
+extern constext txt_tab[];
 extern constext txt_tagged_art[];
 extern constext txt_tagged_thread[];
-extern constext txt_testing_archive[];
 extern constext txt_there_is_no_news[];
 extern constext txt_thread_com[];
 extern constext txt_thread_marked_as_deselected[];
 extern constext txt_thread_marked_as_selected[];
-extern constext txt_thread_not_saved[];
-extern constext txt_thread_page[];
-extern constext txt_thread_resp_page[];
-extern constext txt_thread_saved_to_many[];
 extern constext txt_thread_x_of_n[];
 extern constext txt_threading_arts[];
 extern constext txt_time_default_days[];
-
 extern constext txt_tinrc_defaults[];
 extern constext txt_tinrc_filter[];
 extern constext txt_tinrc_header[];
 extern constext txt_tinrc_info_in_last_line[];
 extern constext txt_tinrc_newnews[];
-extern constext txt_tinrc_post_process_command[];
-
+extern constext txt_tinrc_post_process_view[];
 extern constext txt_toggled_rot13[];
 extern constext txt_toggled_tex2iso[];
 extern constext txt_type_h_for_help[];
 extern constext txt_unlimited_time[];
+extern constext txt_unread[];
 extern constext txt_unsubscribe_pattern[];
 extern constext txt_unsubscribed_num_groups[];
 extern constext txt_unsubscribed_to[];
@@ -921,22 +944,25 @@ extern constext txt_unsubscribing[];
 extern constext txt_untagged_art[];
 extern constext txt_untagged_thread[];
 extern constext txt_unthreading_arts[];
-extern constext txt_uudecoding[];
+extern constext txt_updated[];
+#ifndef HAVE_LIBUU
+#if 0
+	extern constext txt_uudecoding[];
+#endif /* 0 */
+#endif /* !HAVE_LIBUU */
+#ifndef HAVE_UUDECODE
+	extern constext txt_uuencode_not_supported[];
+#endif /* !HAVE_UUDECODE */
 extern constext txt_value_out_of_range[];
+extern constext txt_view_attachment[];
 extern constext txt_warn_art_line_too_long[];
 extern constext txt_warn_blank_subject[];
 extern constext txt_warn_cancel[];
 extern constext txt_warn_encoding_and_external_inews[];
-
-#ifndef HAVE_FASCIST_NEWSADMIN
-	extern constext txt_warn_followup_to_several_groups[];
-	extern constext txt_warn_missing_followup_to[];
-	extern constext txt_warn_not_in_newsrc[];
-	extern constext txt_warn_not_valid_newsgroup[];
-#endif /* !HAVE_FASCIST_NEWSADMIN */
-
 extern constext txt_warn_newsrc[];
 extern constext txt_warn_multiple_sigs[];
+extern constext txt_warn_re_but_no_references[];
+extern constext txt_warn_references_but_no_re[];
 extern constext txt_warn_sig_too_long[];
 extern constext txt_warn_suspicious_mail[];
 extern constext txt_warn_update[];
@@ -949,6 +975,44 @@ extern constext txt_yanking_sub_groups[];
 extern constext txt_yes[];
 extern constext txt_you_have_mail[];
 
+#ifndef DISABLE_PRINTING
+	extern constext txt_help_o[];
+	extern constext txt_print[];
+	extern constext txt_printing[];
+#endif /* !DISABLE_PRINTING */
+
+#ifndef DONT_HAVE_PIPING
+	extern constext txt_help_pipe[];
+	extern constext txt_no_command[];
+#endif /* !DONT_HAVE_PIPING */
+
+#ifndef HAVE_FASCIST_NEWSADMIN
+	extern constext txt_warn_followup_to_several_groups[];
+	extern constext txt_warn_missing_followup_to[];
+	extern constext txt_warn_not_in_newsrc[];
+	extern constext txt_warn_not_valid_newsgroup[];
+#endif /* !HAVE_FASCIST_NEWSADMIN */
+
+#ifdef HAVE_LIBUU
+	extern constext txt_libuu_saved[];
+	extern constext txt_libuu_success[];
+	extern constext txt_libuu_error_missing[];
+	extern constext txt_libuu_error_no_begin[];
+	extern constext txt_libuu_error_no_data[];
+	extern constext txt_libuu_error_unknown[];
+#endif /* HAVE_LIBUU */
+extern constext txt_uu_error_decode[];
+extern constext txt_uu_error_no_end[];
+extern constext txt_uu_success[];
+
+#if defined(NNTP_ABLE) || defined(NNTP_ONLY)
+	extern constext txt_cannot_open_active_file[];
+#endif /* NNTP_ABLE || NNTP_ONLY */
+
+#ifndef NO_SHELL_ESCAPE
+	extern constext txt_shell_escape[];
+#endif /* !NO_SHELL_ESCAPE */
+
 extern int *my_group;
 extern int MORE_POS;
 extern int NOTESLINES;
@@ -956,13 +1020,10 @@ extern int RIGHT_POS;
 extern int _hp_glitch;
 extern int cCOLS;
 extern int cLINES;
-extern int cur_groupnum;
+extern int curr_line;
 extern int debug;
-extern int glob_respnum;
 extern int group_hash[TABLE_SIZE];
-extern int group_top;
 extern int groupname_len;
-extern int index_point;
 extern int iso2asc_supported;
 extern int last_resp;
 extern int max_active;
@@ -974,8 +1035,6 @@ extern int num_headers_to_display;
 extern int num_headers_to_not_display;
 extern int max_subj;
 extern int need_resize;
-extern int note_line;
-extern int note_page;				/* what page we're on */
 extern int num_active;
 extern int num_newnews;
 extern int num_of_killed_arts;
@@ -983,30 +1042,32 @@ extern int num_of_selected_arts;
 extern int num_of_tagged_arts;
 extern int num_save;
 extern int signal_context;
+extern int srch_lineno;
 extern int system_status;
 extern int this_resp;
 extern int thread_basenote;
 extern int tin_errno;
-extern int top;
-extern int top_base;
+extern int top_art;
 extern int xcol;
 extern int xmouse;
 extern int xrow;
 
 extern long *base;
 extern long head_next;
-extern long note_mark[MAX_PAGES];	/* ftells on beginnings of pages */
-extern long mark_body;					/* ftell on beginning of body */
-extern long note_size;
 
 extern signed long int read_newsrc_lines;
 
-extern gid_t real_gid;
-extern gid_t tin_gid;
 extern mode_t real_umask;
 extern pid_t process_id;
-extern uid_t real_uid;
-extern uid_t tin_uid;
+
+extern struct regex_cache strip_re_regex;
+extern struct regex_cache strip_was_regex;
+extern struct regex_cache uubegin_regex;
+extern struct regex_cache uubody_regex;
+extern struct regex_cache url_regex;
+extern struct regex_cache mail_regex;
+extern struct regex_cache news_regex;
+extern struct regex_cache shar_regex;
 
 #ifdef HAVE_COLOR
 	extern struct regex_cache quote_regex;
@@ -1014,61 +1075,50 @@ extern uid_t tin_uid;
 	extern struct regex_cache quote_regex3;
 #endif /* HAVE_COLOR */
 
-extern struct regex_cache strip_re_regex;
-extern struct regex_cache strip_was_regex;
-
 extern struct t_article *arts;
 extern struct t_config tinrc;
 extern struct t_filters glob_filter;
 extern struct t_group *active;
-extern struct t_header note_h;
 extern struct t_newnews *newnews;
 extern struct t_option option_table[];
 extern struct t_posted *posted;
 extern struct t_save *save;
+
 extern t_bool can_post;
-extern t_bool catchup;
-extern t_bool check_any_unread;
 extern t_bool check_for_new_newsgroups;
 extern t_bool cmd_line;
 extern t_bool created_rcdir;
 extern t_bool dangerous_signal_exit; /* TRUE if SIGHUP, SIGTERM, SIGUSR1 */
-#ifdef INDEX_DAEMON
-	extern t_bool delete_index_file;
-#endif /* INDEX_DAEMON */
 extern t_bool disable_gnksa_domain_check;
 extern t_bool disable_sender;
-extern t_bool do_rfc1521_decoding;
 extern t_bool force_reread_active_file;
 extern t_bool got_sig_pipe;
-extern t_bool in_headers; /* colorful headers */
+extern t_bool have_linescroll;
 extern t_bool filtered_articles;
-extern t_bool local_index;
-extern t_bool mail_news;
 extern t_bool list_active;
 extern t_bool newsrc_active;
-extern t_bool note_end;					/* end of article ? */
 extern t_bool no_write;
 extern t_bool post_article_and_exit;
 extern t_bool post_postponed_and_exit;
-extern t_bool purge_index_files;
 extern t_bool read_local_newsgroups_file;
 extern t_bool read_news_via_nntp;
 extern t_bool read_saved_news;
 extern t_bool reconnected_in_last_get_server;
 extern t_bool reread_active_for_posted_arts;
-extern t_bool save_news;
 extern t_bool show_description;
 extern t_bool show_subject;
-extern t_bool space_mode;
-extern t_bool start_any_unread;
 extern t_bool tex2iso_supported;
 extern t_bool batch_mode;
-extern t_bool update_fork;
 extern t_bool verbose;
 extern t_bool xover_supported;
 extern t_bool xref_supported;
 
+extern t_menu selmenu;
+extern t_menu grpmenu;
+extern t_menu *currmenu;
+
+extern t_openartinfo pgart;
+
 #define HIST_SIZE		15
 #define HIST_OTHER		 0
 #define HIST_ART_SEARCH		 1
@@ -1135,47 +1185,12 @@ extern char *input_history[HIST_MAXNUM+1][HIST_SIZE+1];
 #define GNKSA_ADDRTYPE_ROUTE	0
 #define GNKSA_ADDRTYPE_OLDSTYLE	1
 
-/* This fixes ambiguities on platforms that don't distinguish extern case */
-#ifdef CASE_PROBLEM
-#	define txt_help_B	txt_help_BIGB
-#	define txt_help_D	txt_help_BIGD
-#	define txt_help_I	txt_help_BIGI
-#	define txt_help_K	txt_help_BIGK
-#	define txt_help_L	txt_help_BIGL
-#	define txt_help_M	txt_help_BIGM
-#	define txt_help_Q	txt_help_BIGQ
-#	define txt_help_S	txt_help_BIGS
-#	define txt_help_T	txt_help_BIGT
-#	define txt_help_U	txt_help_BIGU
-#	define txt_help_W	txt_help_BIGW
-#	define txt_help_X	txt_help_BIGX
-#	define txt_help_p_S	txt_help_p_BIGS
-#	define txt_help_t_K	txt_help_t_BIGK
-#endif /* CASE_PROBLEM */
-
-#ifdef VMS /* M.St. 15.01.98 */
-	extern constext txt_help_BIGB[];
-	extern constext txt_help_BIGD[];
-	extern constext txt_help_BIGI[];
-	extern constext txt_help_BIGK[];
-	extern constext txt_help_BIGL[];
-	extern constext txt_help_BIGM[];
-	extern constext txt_help_BIGQ[];
-	extern constext txt_help_BIGS[];
-	extern constext txt_help_BIGT[];
-	extern constext txt_help_BIGU[];
-	extern constext txt_help_BIGW[];
-	extern constext txt_help_BIGX[];
-	extern constext txt_help_p_BIGS[];
-	extern constext txt_help_t_BIGK[];
-#endif /* VMS */
-
 #ifndef DONT_HAVE_PIPING
 	extern constext txt_pipe[];
 	extern constext txt_pipe_to_command[];
 	extern constext txt_piping[];
 #else
-#	ifdef VMS /* M.St. 15.01.98 */
+#	ifdef VMS
 		extern constext txt_pipe_to_command[];
 		extern constext txt_piping[];
 #	endif /* VMS */
@@ -1208,43 +1223,32 @@ extern char *input_history[HIST_MAXNUM+1][HIST_SIZE+1];
 
 #ifdef HAVE_METAMAIL
 	extern constext txt_use_mime[];
+	extern constext txt_error_metamail_failed[];
 #endif /* HAVE_METAMAIL */
 
-#if defined(HAVE_METAMAIL) && !defined(INDEX_DAEMON)
-	extern constext txt_error_metamail_failed[];
-#endif /* HAVE_METAMAIL && !INDEX_DAEMON */
-
-#ifdef HAVE_PGP
-	extern char pgp_data[PATH_LEN];
-	extern const char *pgpopts;
+#ifdef HAVE_PGP_GPG
 	extern constext txt_help_ctrl_g[];
 	extern constext txt_pgp_add[];
 	extern constext txt_pgp_mail[];
 	extern constext txt_pgp_news[];
 	extern constext txt_pgp_not_avail[];
 	extern constext txt_pgp_nothing[];
-#endif /* HAVE_PGP */
+#endif /* HAVE_PGP_GPG */
 
 #ifdef HAVE_SYS_UTSNAME_H
 	extern struct utsname system_info;
 #endif /* HAVE_SYS_UTSNAME_H */
 
-#ifdef INDEX_DAEMON
-	extern char group_times_file[PATH_LEN];
-	extern constext txt_cannot_stat_group[];
-	extern constext txt_cannot_stat_index_file[];
-#else
-	extern constext txt_art_deleted[];
-	extern constext txt_art_undeleted[];
-	extern constext txt_intro_page[];
-	extern constext txt_processing_mail_arts[];
-	extern constext txt_processing_saved_arts[];
-#endif /* INDEX_DAEMON */
+extern constext txt_art_deleted[];
+extern constext txt_art_undeleted[];
+extern constext txt_intro_page[];
+extern constext txt_processing_mail_arts[];
+extern constext txt_processing_saved_arts[];
 
-#if !defined(INDEX_DAEMON) && defined(HAVE_MH_MAIL_HANDLING)
+#ifdef HAVE_MH_MAIL_HANDLING
 	extern constext txt_reading_mail_active_file[];
 	extern constext txt_reading_mailgroups_file[];
-#endif /* !defined(INDEX_DAEMON) && defined(HAVE_MH_MAIL_HANDLING) */
+#endif /* HAVE_MH_MAIL_HANDLING */
 
 #ifdef LOCAL_CHARSET
 	extern constext txt_tinrc_local_charset[];
@@ -1273,6 +1277,7 @@ extern char *input_history[HIST_MAXNUM+1][HIST_SIZE+1];
 #endif /* !USE_CURSES */
 
 #ifdef NNTP_ABLE
+	extern constext txt_nntp_ok_goodbye[];
 	extern unsigned short nntp_tcp_port;
 	extern t_bool force_auth_on_conn_open;
 #endif /* NNTP_ABLE */
@@ -1297,66 +1302,33 @@ extern struct opttxt txt_art_marked_deleted;
 extern struct opttxt txt_art_marked_inrange;
 extern struct opttxt txt_art_marked_return;
 extern struct opttxt txt_art_marked_selected;
+extern struct opttxt txt_art_marked_recent;
 extern struct opttxt txt_art_marked_unread;
+extern struct opttxt txt_art_marked_read;
+extern struct opttxt txt_art_marked_killed;
+extern struct opttxt txt_art_marked_read_selected;
 extern struct opttxt txt_groupname_max_length;
 extern struct opttxt txt_show_lines;
 extern struct opttxt txt_show_score;
 extern struct opttxt txt_full_page_scroll;
 extern struct opttxt txt_show_last_line_prev_page;
 extern struct opttxt txt_show_signatures;
+extern struct opttxt txt_hide_uue;
 extern struct opttxt txt_news_headers_to_display;
 extern struct opttxt txt_news_headers_to_not_display;
 extern struct opttxt txt_show_xcommentto;
-extern struct opttxt txt_display_mime_header_asis;
-extern struct opttxt txt_display_mime_allheader_asis;
 extern struct opttxt txt_alternative_handling;
-#ifdef HAVE_COLOR
-	extern struct opttxt txt_quote_regex;
-	extern struct opttxt txt_quote_regex2;
-	extern struct opttxt txt_quote_regex3;
-#endif	/* HAVE_COLOR */
 extern struct opttxt txt_strip_re_regex;
 extern struct opttxt txt_strip_was_regex;
-#ifdef HAVE_METAMAIL
-	extern struct opttxt txt_use_metamail;
-	extern struct opttxt txt_ask_for_metamail;
-#endif	/* HAVE_METAMAIL */
 extern struct opttxt txt_catchup_read_groups;
 extern struct opttxt txt_group_catchup_on_exit;
 extern struct opttxt txt_thread_catchup_on_exit;
 extern struct opttxt txt_confirm_action;
 extern struct opttxt txt_confirm_to_quit;
 extern struct opttxt txt_use_mouse;
-#ifdef HAVE_KEYPAD
-	extern struct opttxt txt_use_keypad;
-#endif	/* HAVE_KEYPAD */
 extern struct opttxt txt_use_getart_limit;
 extern struct opttxt txt_getart_limit;
-#ifdef HAVE_COLOR
-	extern struct opttxt txt_use_color;
-	extern struct opttxt txt_col_normal;
-	extern struct opttxt txt_col_back;
-	extern struct opttxt txt_col_invers_bg;
-	extern struct opttxt txt_col_invers_fg;
-	extern struct opttxt txt_col_text;
-	extern struct opttxt txt_col_minihelp;
-	extern struct opttxt txt_col_help;
-	extern struct opttxt txt_col_message;
-	extern struct opttxt txt_col_quote;
-	extern struct opttxt txt_col_quote2;
-	extern struct opttxt txt_col_quote3;
-	extern struct opttxt txt_col_head;
-	extern struct opttxt txt_col_newsheaders;
-	extern struct opttxt txt_col_subject;
-	extern struct opttxt txt_col_response;
-	extern struct opttxt txt_col_from;
-	extern struct opttxt txt_col_title;
-	extern struct opttxt txt_col_signature;
-	extern struct opttxt txt_word_highlight;
-	extern struct opttxt txt_word_h_display_marks;
-	extern struct opttxt txt_col_markstar;
-	extern struct opttxt txt_col_markdash;
-#endif	/* HAVE_COLOR */
+extern struct opttxt txt_recent_time;
 extern struct opttxt txt_mail_address;
 extern struct opttxt txt_prompt_followupto;
 extern struct opttxt txt_sigfile;
@@ -1372,6 +1344,7 @@ extern struct opttxt txt_advertising;
 extern struct opttxt txt_mm_charset;
 extern struct opttxt txt_post_mime_encoding;
 extern struct opttxt txt_post_8bit_header;
+extern struct opttxt txt_post_process_view;
 extern struct opttxt txt_mail_mime_encoding;
 extern struct opttxt txt_mail_8bit_header;
 extern struct opttxt txt_strip_blanks;
@@ -1399,11 +1372,47 @@ extern struct opttxt txt_mailer_format;
 extern struct opttxt txt_use_mailreader_i;
 extern struct opttxt txt_unlink_article;
 extern struct opttxt txt_keep_posted_articles;
+extern struct opttxt txt_keep_posted_articles_file;
 extern struct opttxt txt_keep_dead_articles;
 extern struct opttxt txt_strip_newsrc;
 extern struct opttxt txt_strip_bogus;
 extern struct opttxt txt_reread_active_file_secs;
 extern struct opttxt txt_auto_reconnect;
 extern struct opttxt txt_cache_overview_files;
+#ifdef HAVE_COLOR
+	extern struct opttxt txt_quote_regex;
+	extern struct opttxt txt_quote_regex2;
+	extern struct opttxt txt_quote_regex3;
+	extern struct opttxt txt_use_color;
+	extern struct opttxt txt_col_normal;
+	extern struct opttxt txt_col_back;
+	extern struct opttxt txt_col_invers_bg;
+	extern struct opttxt txt_col_invers_fg;
+	extern struct opttxt txt_col_text;
+	extern struct opttxt txt_col_minihelp;
+	extern struct opttxt txt_col_help;
+	extern struct opttxt txt_col_message;
+	extern struct opttxt txt_col_quote;
+	extern struct opttxt txt_col_quote2;
+	extern struct opttxt txt_col_quote3;
+	extern struct opttxt txt_col_head;
+	extern struct opttxt txt_col_newsheaders;
+	extern struct opttxt txt_col_subject;
+	extern struct opttxt txt_col_response;
+	extern struct opttxt txt_col_from;
+	extern struct opttxt txt_col_title;
+	extern struct opttxt txt_col_signature;
+	extern struct opttxt txt_word_highlight;
+	extern struct opttxt txt_word_h_display_marks;
+	extern struct opttxt txt_col_markstar;
+	extern struct opttxt txt_col_markdash;
+#endif /* HAVE_COLOR */
+#ifdef HAVE_KEYPAD
+	extern struct opttxt txt_use_keypad;
+#endif /* HAVE_KEYPAD */
+#ifdef HAVE_METAMAIL
+	extern struct opttxt txt_use_metamail;
+	extern struct opttxt txt_ask_for_metamail;
+#endif /* HAVE_METAMAIL */
 
 #endif /* !EXTERN_H */
