@@ -5,13 +5,39 @@
  *  Created   : 1991-04-01
  *  Updated   : 1997-12-31
  *  Notes     : #include files, #defines & struct's
- *  Copyright : (c) Copyright 1991-99 by Iain Lea & Rich Skrenta
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1997-2001 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Iain Lea, Rich Skrenta.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 /*
  * OS specific doda's
  */
@@ -25,24 +51,47 @@
 #	endif /* !TIN_AUTOCONF_H */
 #else
 #	ifndef HAVE_CONFDEFS_H
-#		include	"config.h"
+#		include	"oldconfig.h"
 #	endif /* !HAVE_CONFDEFS_H */
 #endif /* HAVE_CONFIG_H */
 
 
-/* FIXME */
-#if 0
 /*
  * this causes trouble on Linux (forces nameserver lookups even for local
  * connections - this is an (unwanted) feature of getaddrinfo) see also
  * nntplib.c
  */
 /* IPv6 support */
-#if defined(HAVE_GETADDRINFO) && defined(HAVE_GAI_STRERROR)
+#if defined(HAVE_GETADDRINFO) && defined(HAVE_GAI_STRERROR) && defined(ENABLE_IPV6)
 #	define INET6
-#endif /* HAVE_GETADDRINFO && HAVE_GAI_STRERROR */
-#endif /* 0 */
+#endif /* HAVE_GETADDRINFO && HAVE_GAI_STRERROR && ENABLE_IPV6 */
 
+/*
+ * Native Language Support.
+ */
+#ifdef HAVE_LOCALE_H
+#	include <locale.h>
+#endif /* HAVE_LOCALE_H */
+#ifndef HAVE_SETLOCALE
+#	define setlocale(Category, Locale) /* empty */
+#endif /* !HAVE_SETLOCALE */
+
+#define N_(Str) Str
+
+#ifdef ENABLE_NLS
+#	include <libintl.h>
+#	define _(Text) gettext (Text)
+#else
+#	undef bindtextdomain
+#	define bindtextdomain(Domain, Directory) /* empty */
+#	undef textdomain
+#	define textdomain(Domain) /* empty */
+#	define _(Text) Text
+#endif /* ENABLE_NLS */
+
+#ifndef LOCALEDIR
+#	define LOCALEDIR "/usr/share/locale"
+#endif /* !LOCALEDIR */
 
 /*
  * Non-autoconf'able definitions for Amiga Developer Environment (gcc 2.7.2,
@@ -113,8 +162,10 @@ enum resizer { cNo, cYes, cRedraw };
 #		include <types.h>
 #	endif /* !MULTINET */
 #	define FOPEN_OPTS       , "fop=cif"
+#	define JOINPATH			joindir
 #else
 #	define FOPEN_OPTS
+#	define JOINPATH			joinpath
 #endif /* VMS */
 
 #include <stdio.h>
@@ -326,6 +377,15 @@ enum resizer { cNo, cYes, cRedraw };
 #	define isascii(c) (!((c) & ~0177))
 #endif /* !HAVE_ISASCII && !isascii */
 
+
+/*
+ * any pgp/gpp support possible and wanted
+ */
+#if defined(HAVE_PGP) || defined(HAVE_PGPK) || defined(HAVE_GPG)
+#	define HAVE_PGP_GPG 1
+#endif /* HAVE_PGP || HAVE_PGPK || HAVE_GPG */
+
+
 /*
  * Setup support for reading from NNTP
  */
@@ -338,10 +398,6 @@ enum resizer { cNo, cYes, cRedraw };
 #	endif /* !NNTP_INEWS */
 #endif /* NNTP_ABLE || NNTP_ONLY */
 
-#if defined(INDEX_DAEMON) && defined(FORGERY)
-#	undef FORGERY
-#endif /* INDEX_DAEMON && FORGERY */
-
 #define FAKE_NNTP_FP		(FILE *) 9999
 
 /*
@@ -355,22 +411,6 @@ enum resizer { cNo, cYes, cRedraw };
  */
 #define VT_ESCAPE_TIMEOUT	1
 
-/*
- * Index file daemon version of tin. Will create/update index files from cron
- * on NNTP server machine so clients can retreive index file with NNTP XINDEX
- * command from server. Also can be used on machines that just want one copy
- * of all the index files in one place. In this case the normal tin must have
- * access to the index directory (-I dir option) or be setuid news.
- */
-#ifdef INDEX_DAEMON
-#	define LOCK_FILE	"tind.LCK"
-#	undef	HAVE_POLL
-#	undef	HAVE_SELECT
-#	undef	NNTP_ABLE
-#	undef	NNTP_ONLY
-#	undef	NNTP_INEWS
-#endif /* INDEX_DAEMON */
-
 /*
  * Specify News spool & control directories if not running NNTP_ONLY
  * (on machines who can run configure this is not needed)
@@ -466,7 +506,6 @@ enum resizer { cNo, cYes, cRedraw };
 #		define DEFAULT_PRINTER	"copy to PRT:"
 #		define DEFAULT_BBS_PRINTER	"copy to NIL:"
 #		define DEFAULT_SHELL	"newshell"	/* Not Yet Implemented */
-#		define DEFAULT_UUDECODE	"uudecode %s"
 #		define DEFAULT_UNSHAR	"unshar %s"
 #	endif /* M_AMIGA */
 #	ifdef VMS
@@ -476,7 +515,6 @@ enum resizer { cNo, cYes, cRedraw };
 #		define MAILER_FORMAT	"MAIL /SUBJECT=\"%S\" %F MX%%\"%T\""
 #		define DEFAULT_POSTER	"inews %s."
 #		define DEFAULT_PRINTER	"PRINT/DELETE"
-#		define DEFAULT_UUDECODE	"uudecode %s."
 #		define DEFAULT_UNSHAR	"unshar %s."
 #	endif /* VMS */
 #	ifdef M_OS2
@@ -488,7 +526,6 @@ enum resizer { cNo, cYes, cRedraw };
 #		define DEFAULT_POSTER	"postnews %s"
 #		define DEFAULT_PRINTER	"lpt1"
 #		define DEFAULT_SHELL	"cmd.exe"
-#		define DEFAULT_UUDECODE	"uudecode %s"
 #		define DEFAULT_UNSHAR	"unshar %s"
 #	endif /* M_OS2 */
 #	ifdef WIN32
@@ -500,7 +537,6 @@ enum resizer { cNo, cYes, cRedraw };
 #		define DEFAULT_POSTER	"postnews %s"
 #		define DEFAULT_PRINTER	"lpt1"
 #		define DEFAULT_SHELL	"cmd.exe"
-#		define DEFAULT_UUDECODE	"uudecode %s"
 #		define DEFAULT_UNSHAR	"unshar %s"
 #	endif /* WIN32 */
 #	ifdef QNX42
@@ -632,17 +668,58 @@ enum resizer { cNo, cYes, cRedraw };
 #define DEFAULT_MAILDIR	"Mail"
 #define DEFAULT_SAVEDIR	"News"
 
+
+/* MMDF-mailbox seperator */
+#ifndef MMDFHDRTXT
+#	define MMDFHDRTXT "\01\01\01\01\n"
+#endif /* MMDFHDRTXT */
+
+
+/*
+ * all regexps are extended -> # must be quoted!
+ */
 #ifdef HAVE_COLOR
 /* case insensitive */
-#	define DEFAULT_QUOTE_REGEX "^\\s{0,3}([\\]{}>|:)]|\\w{1,3}[>|])(?!-)"
-#	define DEFAULT_QUOTE_REGEX2 "^\\s{0,3}(([\\]{}>|:)]|\\w{1,3}[>|])\\s*){2}(?!-[})>])"
-#	define DEFAULT_QUOTE_REGEX3 "^\\s{0,3}(([\\]{}>|:)]|\\w{1,3}[>|])\\s*){3}"
+#	define DEFAULT_QUOTE_REGEX	"^\\s{0,3}(?:[\\]{}>|:)]|\\w{1,3}[>|])(?!-)"
+#	define DEFAULT_QUOTE_REGEX2	"^\\s{0,3}(?:(?:[\\]{}>|:)]|\\w{1,3}[>|])\\s*){2}(?!-[})>])"
+#	define DEFAULT_QUOTE_REGEX3	"^\\s{0,3}(?:(?:[\\]{}>|:)]|\\w{1,3}[>|])\\s*){3}"
 #endif /* HAVE_COLOR */
 
-/* case sensitive && anchored */
-#define DEFAULT_STRIP_RE_REGEX "(R[eE](\\^\\d+|\\[\\d\\])?|A[wW]|Odp):\\s"
+/* case sensitive && ^-anchored */
+#define DEFAULT_STRIP_RE_REGEX	"(?:R[eE](?:\\^\\d+|\\[\\d\\])?|A[wW]|Odp|Sv):\\s"
 /* case sensitive */
-#define DEFAULT_STRIP_WAS_REGEX ".\\(([Ww]a[rs]|[Bb]y[l]o):.*\\)\\s*$"
+#define DEFAULT_STRIP_WAS_REGEX	".\\((?:[Ww]a[rs]|[Bb]y[l]o):.*\\)\\s*$"
+
+/* case insensitive & ^-anchored */
+#define UUBEGIN_REGEX	"begin\\s+[0-7]{3,4}\\s+"
+/* case sensitive & ^-anchored */
+#define UUBODY_REGEX	"(?:`|.[\\x20-\\x60]{1,61})$"
+
+/* case sensitive & ^-anchored */
+#define SHAR_REGEX	"\\#(?:!\\s?(?:/usr)?/bin/sh|\\s?(?i)this\\sis\\sa\\sshell\\sarchive)"
+
+
+/*
+ * URL related regexs:
+ * add TELNET & WAIS (RFC 1738), IMAP (RFC 2192), NFS (RFC 2224)
+ *     LDAP (RFC 2255), POP (RFC 2384)
+ * add IPv6 (RFC 2732, RFC 2373) support
+ */
+/*
+ * case insensitive
+ * split out ftp (only ftp allows username:passwd@, RFC 1738)
+ */
+#define URL_REGEX "\\b(?:https?|ftp|gopher)://(?:[^:@/]*(?::[^:@/]*)?@)?(?:[^\\W_]+(?:(?:[-.][^\\W_]+)+)?\\.[a-z]{2,5}|localhost|(?:(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?))(?::\\d+)?(?:/[^)\\>\"\\s]*|$|(?=[)\\>\"\\s]))"
+/*
+ * case insensitive
+ * check against RFC 2368
+ */
+#define MAIL_REGEX	"\\b(?:mailto:(?:(?:[-\\w$.+!*'(),;/?:@&=]|(?:%[\\da-f]{2}))+))"
+/*
+ * case insensitive
+ */
+#define NEWS_REGEX	"\\b(?:s?news|nntp):[^\\s@]+[@.][^\\s@]+(?:$|(?=[\\s.>\"]))\\b"
+
 
 #define FILTER_FILE	"filter"
 #define GROUP_TIMES_FILE	"group.times"
@@ -666,6 +743,8 @@ enum resizer { cNo, cYes, cRedraw };
 #define _CONF_ORGANIZATION	"organization"
 #define _CONF_SERVER	"server"
 
+#define SIGDASHES "-- \n"
+
 #ifndef BOOL_H
 #	include "bool.h"
 #endif /* BOOL_H */
@@ -674,10 +753,10 @@ enum resizer { cNo, cYes, cRedraw };
 #include	<pcre.h>
 
 #ifndef MAX
-#	define MAX(a,b)	((a > b) ? a : b)
+#	define MAX(a,b)	(((a) > (b)) ? (a) : (b))
 #endif /* !MAX */
 #ifndef MIN
-#	define MIN(a,b)	((a > b) ? b : a)
+#	define MIN(a,b)	(((a) > (b)) ? (b) : (a))
 #endif /* !MIN */
 
 #ifndef forever
@@ -696,15 +775,15 @@ enum resizer { cNo, cYes, cRedraw };
 /* safe strcpy into fixed-legth buffer */
 #define STRCPY(dst, src)	(dst[sizeof(dst) - 1] = '\0', strncpy(dst, src, sizeof(dst) -1))
 
-#define STRCMPEQ(s1, s2)	(*(s1) == *(s2) && strcmp((s1), (s2)) == 0)
-#define STRNCMPEQ(s1, s2, n)	(*(s1) == *(s2) && strncmp((s1), (s2), n) == 0)
+#define STRCMPEQ(s1, s2)	(strcmp((s1), (s2)) == 0)
+#define STRNCMPEQ(s1, s2, n)	(strncmp((s1), (s2), n) == 0)
 #define STRNCASECMPEQ(s1, s2, n)	(strncasecmp((s1), (s2), n) == 0)
 
-
 #if defined(VMS) || defined(M_AMIGA)
 #	define LEN	512
 #	define PATH_LEN	256
 #endif /* VMS || M_AMIGA */
+
 #if defined(M_OS2) || defined(M_UNIX)
 #	ifndef MAXPATHLEN
 #		define MAXPATHLEN	256
@@ -712,6 +791,7 @@ enum resizer { cNo, cYes, cRedraw };
 #	define LEN	1024
 #	define PATH_LEN	MAXPATHLEN
 #endif /* M_OS2 || M_UNIX */
+
 #if defined(WIN32)
 #	define LEN	1024
 #	define PATH_LEN	MAX_PATH
@@ -725,6 +805,8 @@ enum resizer { cNo, cYes, cRedraw };
 #define TABLE_SIZE	1409
 #define MAX_PAGES	2000	/* maximum article pages */
 
+#define DAY	(60*60*24)		/* Seconds in a day */
+
 #define ctrl(c)	((c) & 0x1F)
 
 #ifndef DEFAULT_ISO2ASC
@@ -743,12 +825,18 @@ enum resizer { cNo, cYes, cRedraw };
 #ifndef ART_MARK_SELECTED
 #	define ART_MARK_SELECTED	'*'	/* used to show that an art was auto selected */
 #endif /* !ART_MARK_SELECTED */
+#ifndef ART_MARK_RECENT
+#	define ART_MARK_RECENT	'o'	/* used to show that an art is fresh */
+#endif /* !ART_MARK_RECENT */
 #ifndef ART_MARK_READ
 #	define ART_MARK_READ	' '	/* used to show that an art was not read or seen */
 #endif /* !ART_MARK_READ */
 #ifndef ART_MARK_READ_SELECTED
-#	define ART_MARK_READ_SELECTED ':'	/* used to show that an read art is hot */
+#	define ART_MARK_READ_SELECTED ':'	/* used to show that an read art is hot (kill_level >0) */
 #endif /* !ART_MARK_READ_SELECTED */
+#ifndef ART_MARK_KILLED
+#	define ART_MARK_KILLED 'K'		/* art has been killed (kill_level >0) */
+#endif /*! ART_MARK_KILLED */
 #ifndef ART_MARK_DELETED
 #	define ART_MARK_DELETED	'D'	/* art has been marked for deletion (mailgroup) */
 #endif /* !ART_MARK_DELETED */
@@ -803,6 +891,8 @@ enum resizer { cNo, cYes, cRedraw };
  */
 #define MAX_MARK		3
 
+/* Line number (starting at 0) of 1st non-header data on the screen */
+/* ie, size of header */
 #define INDEX_TOP	2
 
 #define GROUP_MATCH(s1, pat, case)		(wildmat (s1, pat, case))
@@ -812,6 +902,12 @@ enum resizer { cNo, cYes, cRedraw };
 
 #define IGNORE_ART(i)	((tinrc.kill_level != KILL_THREAD && arts[i].killed) || (arts[i].thread == ART_EXPIRED))
 
+/*
+ * Is this part text/plain ?
+ */
+#define IS_PLAINTEXT(x) \
+			(x->type == TYPE_TEXT && strcmp("plain", x->subtype) == 0)
+
 /* TRUE if basenote has responses */
 #define HAS_FOLLOWUPS(i)	(arts[base[i]].thread != -1)
 
@@ -827,19 +923,19 @@ enum resizer { cNo, cYes, cRedraw };
 /*
  * Often used macro to point to the group we are currenty in
  */
-#define CURR_GROUP	(active[my_group[cur_groupnum]])
+#define CURR_GROUP	(active[my_group[selmenu.curr]])
 
 /*
  * Defines an unread group
  */
 #define UNREAD_GROUP(i)		(!active[my_group[i]].bogus && active[my_group[i]].newsrc.num_unread > 0)
+
 /*
- * Some informational message are only shown if we're running in
- * the background or some other non-curses mode
+ * Expands to singlar/plural version of string
  */
-#define INTERACTIVE	(!batch_mode || update_fork)
-/* Obselete now, I think */
-#define INTERACTIVE2	((cmd_line && !(batch_mode || verbose)) || (batch_mode && update_fork))
+#define PLURAL(x,y)			((x == 1) ? _(y##_singular) : _(y##_plural))
+
+#define POST_PROC_TYPE(x)	(ch_post_process[x])
 
 /*
  * News/Mail group types
@@ -939,11 +1035,6 @@ enum resizer { cNo, cYes, cRedraw };
 #define POST_PROC_NONE		0
 #define POST_PROC_SHAR		1
 #define POST_PROC_UUDECODE	2
-#define POST_PROC_UUD_LST_ZOO	3
-#define POST_PROC_UUD_EXT_ZOO	4
-#define POST_PROC_UUD_LST_ZIP	5
-#define POST_PROC_UUD_EXT_ZIP	6
-
 
 /*
  * used in art.c
@@ -1015,14 +1106,15 @@ enum resizer { cNo, cYes, cRedraw };
  * -1 is kind of overloaded as an error from which_thread() and other functions
  * where we wish to return to the next level up
  */
-#define GRP_RETURN		-1	/* Stop reading group ('T' command) -> return to selection screen */
+#define GRP_RETURN		-1	/* Pager 'T' command only -> return to selection screen */
 #define GRP_QUIT		-2	/* Set by 'Q' when coming all the way out */
-#define GRP_NEXTUNREAD		-3	/* goto next unread group */
-#define GRP_NEXT		-4	/* (catchup) & goto next group */
+#define GRP_NEXTUNREAD	-3	/* (After catchup) goto next unread item */
+#define GRP_NEXT		-4	/* (After catchup) move to next item */
 #define GRP_ARTFAIL		-5	/* show_page() only. Failed to get into the article */
-#define GRP_KILLED		-6	/* Thread was killed at pager level */
-#define GRP_GOTOTHREAD	-7	/* show_page() only. Enter thread menu */
+#define GRP_KILLED		-6	/* ?? Thread was killed at pager level */
+#define GRP_GOTOTHREAD	-7	/* show_page() only. Goto thread menu */
 #define GRP_ENTER		-8	/* New group is set, spin in read_groups() */
+#define GRP_EXIT		-9	/* Normal return to higher level */
 
 #ifndef EXIT_SUCCESS
 #	define EXIT_SUCCESS	0	/* Successful exit status */
@@ -1224,14 +1316,6 @@ typedef unsigned char	t_bitmap;
  */
 #define MSGID_HASH_SIZE		2609
 
-/*
- * These will probably go away when filtering is rewritten
- * Easier access to hashed msgids. Note that in REFS(), y must be free()d
- * msgid is mandatory in an article and cannot be NULL
- */
-#define MSGID(x)			(x->refptr->txt)
-#define REFS(x,y)			((y = get_references(x->refptr->parent)) ? y : "")
-
 /*
  *	struct t_msgid - message id
  */
@@ -1263,7 +1347,6 @@ struct t_article
 {
 	long artnum;			/* Article number in spool directory for group */
 	char *subject;			/* Subject: line from mail header */
-/* t_article.subject is casted to (int *) in art.c :-( */
 	char *from;			/* From: line from mail header (address) */
 	char *name;			/* From: line from mail header (full name) */
 	int gnksa_code;			/* From: line from mail header (GNKSA error code) */
@@ -1273,7 +1356,7 @@ struct t_article
 	char *msgid;			/* Message-ID: unique message identifier */
 	char *refs;			/* References: article reference id's */
 	struct t_msgid *refptr;		/* Pointer to us in the reference tree */
-	int lines;			/* Lines: number of lines in article */
+	int line_count;			/* Lines: number of lines in article */
 	char *archive;			/* Archive-name: line from mail header */
 	char *part;			/* part no. of archive */
 	char *patch;			/* patch no. of archive */
@@ -1303,12 +1386,17 @@ struct t_attribute
 #ifndef DISABLE_PRINTING
 	char *printer;				/* printer command & parameters */
 #endif /* !DISABLE_PRINTING */
-	char *quick_kill_scope; 		/* quick filter kill scope */
+	char *quick_kill_scope;			/* quick filter kill scope */
 	char *quick_select_scope;		/* quick filter select scope */
 	char *mailing_list;			/* mail list email address */
 	char *x_headers;			/* extra headers for message header */
 	char *x_body;				/* bolierplate text for message body */
 	char *from;				/* from line */
+	char *news_quote_format;		/* another way to begin a posting format */
+	char *quote_chars;			/* string to precede quoted text on each line */
+#ifdef HAVE_ISPELL
+	char *ispell;			/* path to ispell and options */
+#endif /* HAVE_ISPELL */
 	unsigned global:1;			/* global/group specific */
 	unsigned quick_kill_header:3;		/* quick filter kill header */
 	unsigned quick_kill_expire:1;		/* quick filter kill limited/unlimited time */
@@ -1327,14 +1415,8 @@ struct t_attribute
 						   3=from descend, 4=from ascend,
 						   5=date descend, 6=date ascend,
 						   7=score descend, 8=score ascend */
-	unsigned int post_proc_type:4;		/* 0=none, 1=shar, 2=uudecode,
-						   3=uud & list zoo, 4=uud & ext zoo*/
+	unsigned int post_proc_type:2;		/* 0=none, 1=shar, 2=uudecode */
 	unsigned int x_comment_to:1;		/* insert X-Comment-To: in Followup */
-	char *news_quote_format;		/* another way to begin a posting format */
-	char *quote_chars;			/* string to precede quoted text on each line */
-#ifdef HAVE_ISPELL
-	char *ispell;			/* path to ispell and options */
-#endif /* HAVE_ISPELL */
 };
 
 /*
@@ -1364,7 +1446,7 @@ struct t_group
 	long xmax;				/* max. article number */
 	long xmin;				/* min. article number */
 	unsigned int type:4;			/* grouptype - newsgroup / mailbox / savebox */
-	unsigned int inrange:4;			/* 1 = group selected via # range command */
+	t_bool inrange:1;			/* TRUE if group selected via # range command */
 	t_bool read_during_session:1;		/* TRUE if group entered during session */
 	t_bool art_was_posted:1;		/* TRUE if art was posted to group */
 	t_bool subscribed:1;			/* TRUE if subscribed to group */
@@ -1374,9 +1456,6 @@ struct t_group
 	struct t_newsrc newsrc;				/* newsrc bitmap specific info. */
 	struct t_attribute *attribute;	/* group specific attributes */
 	struct t_filters *glob_filter;	/* points to filter array */
-#ifdef INDEX_DAEMON
-	time_t last_updated_time;		/* last time group dir was changed */
-#endif /* INDEX_DAEMON */
 };
 
 /*
@@ -1472,52 +1551,21 @@ struct regex_cache {
 	pcre_extra *extra;
 };
 
-struct t_header_list
-{
-	char header[HEADER_LEN];
-	char content[HEADER_LEN];
-	struct t_header_list *next;
-};
-
-struct t_header
-{
-	char from[HEADER_LEN];		/* From: */
-	char path[HEADER_LEN];		/* Path: */
-	char date[HEADER_LEN];		/* Date: */
-	char subj[HEADER_LEN];		/* Subject: */
-	char org[HEADER_LEN];		/* Organization: */
-	char newsgroups[HEADER_LEN];	/* Newsgroups: */
-	char messageid[HEADER_LEN];	/* Message-ID: */
-	char references[HEADER_LEN];	/* References: */
-	char distrib[HEADER_LEN];	/* Distribution: */
-	char keywords[HEADER_LEN];	/* Keywords: */
-	char summary[HEADER_LEN];	/* Summary: */
-	char followup[HEADER_LEN];	/* Followup-To: */
-	char mimeversion[HEADER_LEN];	/* Mime-Version: */
-	char contenttype[HEADER_LEN];	/* Content-Type: */
-	char contentenc[HEADER_LEN];	/* Content-Transfer-Encoding: */
-	char ftnto[HEADER_LEN];		/* Old X-Comment-To: (Used by FIDO) */
-	char authorids[HEADER_LEN];	/* Author-IDs: (USEFOR, 2nd Son of 1036) */
-	struct t_header_list *persist;	/* P-ersistent headers (USEFOR, 2nd Son of 1036) */
-};
-
 struct t_save
 {
-	char *subject;
-	char *dir;
-	char *file;
-	char *archive;
-	char *part;
-	char *patch;
-	int index;
-	int saved;
-	t_bool is_mailbox;
+	char *path;
+	char *file;					/* => file part of *path */
+	struct t_article *artptr;	/* => article in arts[] */
+	t_bool saved;				/* Set if saved okay */
+	t_bool is_mailbox;			/* Set if path is a mailbox */
 };
 
+#ifndef USE_CURSES
 struct t_screen
 {
 	char *col;
 };
+#endif /* !USE_CURSES */
 
 struct t_posted
 {
@@ -1539,6 +1587,7 @@ struct t_art_stat
 	int selected_seen;	/* selected and seen */
 	char art_mark;		/* mark to use for this thread - not used for groups */
 	int score;		/* maximum score */
+	time_t time;		/* latest time */
 };
 
 
@@ -1578,31 +1627,72 @@ struct t_option {
 	struct opttxt *txt;	/* ptr to information / help on option */
 };
 
+/*
+ * Multipart article detection
+ */
+typedef struct {
+	char *subject;
+	int subject_compare_len;
+	int part_number;
+	int total;
+	int base_index;
+} MultiPartInfo;
+
+
+/*
+ * Key information about current menu screen
+ */
+typedef struct {
+	int curr;					/* Current cursor pos (cur_groupnum, index_point, thread_index_point) */
+	int max;					/* Max # on current menu (group_top, top_base, top_thread) */
+	int first;					/* First # on current menu */
+	int last;					/* Last # on current menu (first,last_*_on_screen) */
+	void (*redraw) (void);		/* Redraw function */
+	void (*draw_arrow) (void);	/* Arrow draw */
+} t_menu;
+
+
+/*
+ * Packet of data needed to enter pager
+ */
+typedef struct {
+	int art;
+	t_bool ignore_unavail;
+} t_pagerinfo;
+
+
 /*
  * Time functions.
  */
 typedef struct _TIMEINFO
 {
-	time_t	time;
-	long	usec;
-	long	tzone;
+	time_t time;
+	long usec;
+	long tzone;
 } TIMEINFO;
 
-#if 0		/* Does anyone know what this was going to do ? */
-/*
- * Used for detecting new groups when reading news locally. It's easy to be
- * confused by arrays of pointers to pointers, so typedef's are used for the
- * first level pointers to keep it clearer.
- */
-struct t_notify
-{
-	char *name;
-	int visited;
-};
 
-typedef struct t_group *group_p;
-typedef struct t_notify *notify_p;
-#endif /* 0 */
+/*
+ * mailcap fields
+ * the x-token field is missing, we would need something like
+ * struct t_xtoken { char *xtoken; t_xtoken *next; } for that...
+ */
+typedef struct {
+	char *type;		/* content-type, mandatory */
+	char *command;	/* view-command, mandatory */
+	char *compose;
+	char *composetyped;
+	char *description;
+	char *edit;
+	char *nametemplate;
+	char *print;
+	char *test;
+	char *x11bitmap;
+	int textualnewlines;
+	t_bool needsterminal;
+	t_bool copiousoutput;
+} t_mailcap;
+
 
 /*
  * Determine signal return type
@@ -1616,14 +1706,14 @@ typedef struct t_notify *notify_p;
  */
 #ifdef HAVE_COMPTYPE_VOID
 #	ifdef __STDC__
-		typedef const void t_comptype;
+		typedef const void *t_comptype;
 #	else
-		typedef void t_comptype;
+		typedef void *t_comptype;
 #	endif /* __STDC__ */
 #endif /* HAVE_COMPTYPE_VOID */
 
 #ifdef HAVE_COMPTYPE_CHAR
-	typedef char t_comptype;
+	typedef char *t_comptype;
 #endif /* HAVE_COMPTYPE_CHAR */
 
 #ifdef M_OS2
@@ -1834,7 +1924,14 @@ extern void joinpath (char *result, char *dir, char *file);
 #define SIZEOF(array)	((int)(sizeof array / sizeof array[0]))
 
 #define FreeIfNeeded(p)	if (p != (char *)0) free((char *)p)
-#define FreeAndNull(p)	if (p != (char *)0) { free((char *)p); p = (char *)0; }
+
+#if 0
+#	define FreeAndNull(p)	if (p != (char *)0) { free((char *)p); p = (char *)0; }
+#else
+#	define FreeAndNull(p)	if (p != NULL) { free((void *)p); p = NULL; }
+#endif /* 0 */
+
+#define BlankIfNull(p)	((p) ? (p) : "")
 
 #define my_group_find(x)	add_my_group(x, FALSE)
 #define my_group_add(x)	add_my_group(x, TRUE)
@@ -1973,8 +2070,6 @@ typedef void (*BodyPtr) (char *, FILE *, int);
 #	endif /* !ferror */
 #endif /* __DECC */
 
-#define IS_PLURAL(x)	(x != 1 ? txt_plural : "")
-
 /* FIXME - check also for tmpfile() and provide fallback functions */
 #ifdef HAVE_TEMPNAM
 #	define my_tempnam(a,b)	tempnam(a,b)
@@ -2051,14 +2146,40 @@ extern struct tm *localtime(time_t *);
 #endif /* MSS */
 
 /* libcanlock */
-#if defined(USE_CANLOCK) && !defined(INDEX_DAEMON)
-#  include "../libcanlock/canlock.h"
-#endif /* USE_CANLOCK && !INDEX_DAEMON */
+#ifdef USE_CANLOCK
+#	include "../libcanlock/canlock.h"
+#endif /* USE_CANLOCK */
 
-#ifndef M_AMIGA  /* FXIME: use autoconf here to detect if system has st_mtime */
-# 	define file_changed(file)  file_mtime(file)
+/* FXIME: use autoconf here to detect if stat(2)-structure has st_mtime */
+#ifndef M_AMIGA
+#	define FILE_CHANGED(file)  file_mtime(file)
 #else
-# 	define file_changed(file)  file_size(file)
+#	define FILE_CHANGED(file)  file_size(file)
 #endif /* M_AMIGA */
 
+/* snprintf(), vsnprintf() */
+#ifndef HAVE_SNPRINTF
+#	define snprintf	plp_snprintf
+#endif /* HAVE_SNPRINTF */
+#ifndef HAVE_VSNPRINTF
+#	define vsnprintf	plp_vsnprintf
+#endif /* HAVE_VSNPRINTF */
+
+/* GCC-specific attributes */
+#if defined(__GNUC__) && !defined(__APPLE_CC__)
+#	define UNUSED(x) x __attribute__((unused))
+#else
+#	define UNUSED(x) x
+#endif /* __GNUC__ && !__APPLE_CC__ */
+
+/* can we use mblen()? */
+#if defined(HAVE_MBLEN) && defined(HAVE_SETLOCALE) && defined(MB_CUR_MAX) && !defined (NO_LOCALE)
+#	define ENABLE_MBLEN 1
+#endif /* HAVE_MBLEN && HAVE_SETLOCALE && MB_CUR_MAX && !NO_LOCALE */
+
+/* init_selfinfo() needs MM_CHARSET */
+#ifndef MM_CHARSET
+#	define MM_CHARSET "US-ASCII"
+#endif /* !MM_CHARSET */
+
 #endif /* !TIN_H */
