@@ -5,13 +5,39 @@
  *  Created   : 1993-12-10
  *  Updated   : 1994-02-28
  *  Notes     : ISO to ascii charset conversion routines
- *  Copyright : (c) Copyright 1991-99 by Markus Kuhn & Tilman Burmester
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1993-2001 Markus Kuhn <mgk25@cl.cam.ac.uk>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Markus Kuhn.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
@@ -114,8 +140,8 @@ static constext *const iso2asc[NUM_ISO_TABLES][256-ISO_EXTRA] =
  * german tex style to latin1 conversion (by root@aspic, 12/04/93)
  */
 
-#define	TEX_SUBST	15
-#define	SPACES		"                                                                                                         "
+#define TEX_SUBST	15
+#define SPACES		"                                                                                                         "
 
 static const char *const tex_from[TEX_SUBST] =
 {
@@ -131,7 +157,7 @@ static const char *const tex_to[TEX_SUBST] =
  */
 
 void
-ConvertIso2Asc (
+convert_iso2asc (
 	char *iso,
 	char *asc,
 	int t)
@@ -216,13 +242,14 @@ ConvertIso2Asc (
 
 
 void
-ConvertTeX2Iso (
+convert_tex2iso (
 	char *from,
 	char *to)
 {
-	size_t len, col, spaces;	/* length of from, col counter, spaces to add */
-	size_t subst_len;
 	int i, ex;
+	int spaces; /* spaces to add */
+	size_t len, col;	/* length of from, col counter */
+	size_t subst_len;
 
 	*to = '\0';
 	len = strlen (from);
@@ -253,33 +280,23 @@ ConvertTeX2Iso (
 }
 
 /*
- * Check for german TeX encoding
+ * Check for german TeX encoding in file open on fp
  */
-
 t_bool
-iIsArtTexEncoded (
-	long art,
-	char *group_path)
+is_art_tex_encoded (
+	FILE *fp)
 {
 	char line[LEN];
-	FILE *fp;
 	int i, len;
 	t_bool body = FALSE;
 
-	/*
-	 * TODO: This is a farce. Reread the whole art !!
-	 *       should be done as part of single pass when article is pulled down
-	 */
-	if ((fp = open_art_fp ((char *)group_path, art, 0, TRUE)) == (FILE *) 0)
-		return FALSE;
+	rewind (fp);
 
 	while (fgets (line, (int) sizeof(line), fp) != (char *) 0) {
-		if (line[0] == '\n' && !body) {
+		if (line[0] == '\n' && !body)
 			body = TRUE;
-		} else {
-			if (!body)
-				continue;
-		}
+		else if (!body)
+			continue;
 
 		i = 0;
 
@@ -294,45 +311,67 @@ iIsArtTexEncoded (
 		len = strlen (line) - 1;
 		for (i = 1; i < len; i++) {
 			if (((line[i] == '\\') || (line[i] == '\"')) &&
-			    (isalnum((unsigned char)line[i-1])) && (isalnum((unsigned char)line[i+1]))) {
-				fclose (fp);
+							(isalnum((unsigned char)line[i-1])) &&
+							(isalnum((unsigned char)line[i+1])))
 				return TRUE;
-			}
 		}
 	}
-	fclose (fp);
 
 	return FALSE;
 }
 
 
 /*
- *  Replace all non printable characters by '?'
+ * Replace all non printable characters by '?'
  */
 void
-Convert2Printable (
-	char *buf)
-{
-	unsigned char *c;
-
-	for (c= (unsigned char *)buf; *c; c++) {
-		if (!my_isprint(*c))
-			*c = '?';
-	}
-}
-
-/*
- *  Same as Convert2Printable() but allows Backspace (ASCII 8), TAB (ASCII 9),
- *  and LineFeed (ASCII 12) according to son-of-rfc 1036 section 4.4
- */
-void
-ConvertBody2Printable (
+convert_to_printable (
 	char *buf)
 {
 	unsigned char *c;
+#ifdef ENABLE_MBLEN
+	int t_len = 0;
+#endif /* ENABLE_MBLEN */
 
 	for (c = (unsigned char *)buf; *c; c++) {
-		if (!(my_isprint(*c) || *c==8 || *c==9 || *c==12))
+#ifdef ENABLE_MBLEN
+		if (!my_isprint(*c) && (t_len = mblen(c, MAX(2,MB_CUR_MAX))) <= 1)
 			*c = '?';
+		while (--t_len > 0)
+			c++;
+#else
+		if (!my_isprint(*c))
+			*c = '?';
+#endif /* ENABLE_MBLEN */
 	}
 }
+
+
+#if 1
+/*
+ * Same as convert_to_printable() but allows Backspace (ASCII 8), TAB (ASCII
+ * 9), and FormFeed (ASCII 12) according to son of RFC 1036 section 4.4;
+ * LineFeed (ASCII 10) and CarriageReturn (ASCII 13) are allowed, too.
+ */
+void
+convert_body2printable (
+	char *buf)
+{
+	unsigned char *c;
+#	ifdef ENABLE_MBLEN
+	int t_len = 0;
+#	endif /* ENABLE_MBLEN */
+
+	for (c = (unsigned char *)buf; *c; c++) {
+#	ifdef ENABLE_MBLEN
+		if (!(my_isprint(*c) || *c == 8 || *c == 9 || *c == 10 || *c == 12 || *c == 13) && (t_len = mblen(c, MAX(2,MB_CUR_MAX))) <= 1)
+			*c = '?';
+		while (--t_len > 0)
+			c++;
+#	else
+		if (!(my_isprint(*c) || *c == 8 || *c == 9 || *c == 10 || *c == 12 || *c == 13))
+			*c = '?';
+#	endif /* ENABLE_MBLEN */
+	}
+}
+#endif /* 1 */
