@@ -5,16 +5,41 @@
  *  Author    : Roland Rosenfeld <roland@spinnaker.rhein.de>
  *              Giuseppe De Marco <gdm@rebel.net> (light-colors)
  *              Julien Oster <fuzzy@cu8.cum.de> (word highlighting)
- *              T.Dickey <dickey@clark.net> (curses support)
+ *              T.Dickey <dickey@herndon4.his.com> (curses support)
  *  Created   : 1995-06-02
  *  Updated   : 1996-12-15
  *  Notes     : This are the basic function for ansi-color
  *              and word highlighting
- *  Copyright : (c) 1995-99 by Olaf Kalzua and Roland Rosenfeld
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1995-2001 Roland Rosenfeld <roland@spinnaker.rhein.de>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Roland Rosenfeld.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #ifndef TIN_H
@@ -29,7 +54,7 @@
 
 #ifdef HAVE_COLOR
 
-#define MIN_COLOR -1	/* -1 is default, otherwise 0-7 or 0-15 */
+#	define MIN_COLOR -1	/* -1 is default, otherwise 0-7 or 0-15 */
 
 int default_fcol = 7;
 int default_bcol = 0;
@@ -42,10 +67,11 @@ static int current_bcol = 0;
  * local prototypes
  */
 static t_bool check_valid_mark (const char *s);
-static void put_mark_char (int c, FILE *stream, t_bool signature);
 static void color_fputs (const char *s, FILE *stream, int color, t_bool signature);
+static void put_mark_char (int c, FILE *stream, t_bool signature);
 
-#ifdef USE_CURSES
+
+#	ifdef USE_CURSES
 static void
 set_colors (
 	int fcolor,
@@ -59,12 +85,12 @@ set_colors (
 	} *list;
 	static int nextpair;
 
-#	ifndef HAVE_USE_DEFAULT_COLORS
+#		ifndef HAVE_USE_DEFAULT_COLORS
 	if (fcolor < 0)
 		fcolor = default_fcol;
 	if (bcolor < 0)
 		bcolor = default_bcol;
-#	endif /* !HAVE_USE_DEFAULT_COLORS */
+#		endif /* !HAVE_USE_DEFAULT_COLORS */
 	if (cmd_line || !use_color || !has_colors()) {
 		current_fcol = default_fcol;
 		current_bcol = default_bcol;
@@ -72,7 +98,7 @@ set_colors (
 		chtype attribute = A_NORMAL;
 		int pair = 0;
 
-		TRACE(("set_colors (%d, %d)", fcolor, bcolor))
+		TRACE(("set_colors (%d, %d)", fcolor, bcolor));
 
 		/* fcolor/bcolor may be negative, if we're using ncurses
 		 * function use_default_colors().
@@ -113,24 +139,25 @@ set_colors (
 	}
 }
 
-void refresh_color (
+void
+refresh_color (
 	void)
 {
 	set_colors(current_fcol, current_bcol);
 }
-#endif /* USE_CURSES */
+#	endif /* USE_CURSES */
 
 /* setting foreground-color */
 void
 fcol (
 	int color)
 {
-	TRACE(("fcol(%d) %s", color, txt_colors[color-MIN_COLOR]))
+	TRACE(("fcol(%d) %s", color, txt_colors[color-MIN_COLOR]));
 	if (use_color) {
 		if (color >= MIN_COLOR && color <= MAX_COLOR) {
-#ifdef USE_CURSES
+#	ifdef USE_CURSES
 			set_colors(color, current_bcol);
-#else
+#	else
 			int bold;
 			if (color < 0)
 				color = default_fcol;
@@ -138,14 +165,14 @@ fcol (
 			my_printf("\033[%d;%dm", bold, ((color & 7) + 30));
 			if (!bold)
 				bcol(current_bcol);
-#endif /* USE_CURSES */
+#	endif /* USE_CURSES */
 			current_fcol = color;
 		}
 	}
-#ifdef USE_CURSES
+#	ifdef USE_CURSES
 	else
 		set_colors(default_fcol, default_bcol);
-#endif /* USE_CURSES */
+#	endif /* USE_CURSES */
 }
 
 /* setting background-color */
@@ -153,23 +180,23 @@ void
 bcol (
 	int color)
 {
-	TRACE(("bcol(%d) %s", color, txt_colors[color-MIN_COLOR]))
+	TRACE(("bcol(%d) %s", color, txt_colors[color-MIN_COLOR]));
 	if (use_color) {
 		if (color >= MIN_COLOR && color <= MAX_BACKCOLOR) {
-#ifdef USE_CURSES
+#	ifdef USE_CURSES
 			set_colors(current_fcol, color);
-#else
+#	else
 			if (color < 0)
 				color = default_bcol;
 			my_printf("\033[%dm", (color + 40));
-#endif /* USE_CURSES */
+#	endif /* USE_CURSES */
 			current_bcol = color;
 		}
 	}
-#ifdef USE_CURSES
+#	ifdef USE_CURSES
 	else
 		set_colors(default_fcol, default_bcol);
-#endif /* USE_CURSES */
+#	endif /* USE_CURSES */
 }
 
 
@@ -228,10 +255,9 @@ color_fputs (
 	t_bool signature)
 {
 	const char *p;
-	const char* eos = strchr(s, '\0');
 	t_bool hilite = FALSE;
 
-	for (p=s; p < eos; p++) {
+	for (p = s; *p; p++) {
 		if (*p == '*' || *p == '_') {
 			if (! hilite) {
 				if ((p == s || !isgraph((unsigned char)p[-1]))
@@ -251,52 +277,50 @@ color_fputs (
 		}
 	}
 }
+#endif /* HAVE_COLOR */
+
 
 /*
  * Output a line of text to the screen with colour if needed
- * word highlights, signatures and 'quote' lines will be coloured
+ * word highlights, signatures etc will be highlighted
  */
 void
-print_color (
-	char *str,
-	t_bool signature)
+draw_pager_line (
+	const char *str,
+	int flags)
 {
+#ifdef HAVE_COLOR
 	int color = tinrc.col_text;
 
 	if (use_color) {
-		if (signature) {
+		if (flags & C_SIG) {
 			fcol (tinrc.col_signature);
 			color = tinrc.col_signature;
-		} else if (in_headers) {
+		} else if (flags & (C_HEADER | C_ATTACH | C_UUE)) {
 			color = tinrc.col_newsheaders;
 			fcol (tinrc.col_newsheaders);
 		} else {
-			if (quote_regex3.re) {
-				if (pcre_exec (quote_regex3.re, quote_regex3.extra, str, strlen(str), 0, 0, NULL, 0) >= 0) {
-					fcol (tinrc.col_quote3);
-					color = tinrc.col_quote3;
-				} else if (quote_regex2.re) {
-					if (pcre_exec (quote_regex2.re, quote_regex2.extra, str, strlen(str), 0, 0, NULL, 0) >= 0) {
-						fcol (tinrc.col_quote2);
-						color = tinrc.col_quote2;
-					} else if (quote_regex.re) {
-						if (pcre_exec (quote_regex.re, quote_regex.extra, str, strlen(str), 0, 0, NULL, 0) >= 0) {
-							fcol (tinrc.col_quote);
-							color = tinrc.col_quote;
-						} else
-							fcol (tinrc.col_text);
-					}
-				}
-			}
+			if (flags & C_QUOTE3) {
+				fcol (tinrc.col_quote3);
+				color = tinrc.col_quote3;
+			} else if (flags & C_QUOTE2) {
+				fcol (tinrc.col_quote2);
+				color = tinrc.col_quote2;
+			} else if (flags & C_QUOTE1) {
+				fcol (tinrc.col_quote);
+				color = tinrc.col_quote;
+			} else
+				fcol (tinrc.col_text);
 		}
 	}
 
 	if (word_highlight && use_color)
-		color_fputs(str, stdout, color, signature);
+		color_fputs(str, stdout, color, (flags & C_SIG));
 	else
-		my_fputs(str, stdout);
-#	ifndef USE_CURSES
-	my_fputs(cCRLF, stdout);
-#	endif /* !USE_CURSES */
-}
 #endif /* HAVE_COLOR */
+		my_fputs(str, stdout);
+
+#ifndef USE_CURSES
+	my_fputs(cCRLF, stdout);
+#endif /* !USE_CURSES */
+}
