@@ -5,13 +5,39 @@
  *  Created   : 1991-04-01
  *  Updated   : 1994-12-24
  *  Notes     : debug routines
- *  Copyright : (c) Copyright 1991-99 by Iain Lea
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Iain Lea.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
@@ -28,9 +54,7 @@ int debug;
 /*
  * Local prototypes
  */
-static void debug_print_group_hash (void);
 static void debug_print_attributes (struct t_attribute *attr, FILE *fp);
-static void debug_print_base (void);
 static void debug_print_filter (FILE *fp, int num, struct t_filter *the_filter);
 #	if 0
 	static void debug_print_active_hash (void);
@@ -104,7 +128,7 @@ debug_print_arts (
 	if (debug != 2)
 		return;
 
-	for (i = 0; i < top; i++)	/* for each group */
+	for (i = 0; i < top_art; i++)	/* for each group */
 		debug_print_header (&arts[i]);
 }
 
@@ -127,7 +151,7 @@ debug_print_header (
 			bool_unparse(s->killed),
 			bool_unparse(s->selected));
 		fprintf (fp,"subj=[%-38s]\n", s->subject);
-		fprintf (fp,"date=[%ld]  from=[%s]  name=[%s]\n", s->date, s->from,
+		fprintf (fp,"date=[%ld]  from=[%s]  name=[%s]\n", (long)s->date, s->from,
 			(s->name ? s->name : ""));
 		fprintf (fp,"msgid=[%s]  refs=[%s]\n",
 			(s->msgid ? s->msgid : ""),
@@ -170,21 +194,10 @@ debug_save_comp (
 	if ((fp = fopen (file, "a+")) != (FILE *) 0) {
 		for (i = 0 ; i < num_save ; i++) {
 
-			fprintf (fp,"subj=[%-38s]\n", save[i].subject);
-			fprintf (fp,"dir=[%s]  file=[%s]\n", save[i].dir, save[i].file);
-			if (save[i].archive)
-				fprintf (fp, "arch=[%-38s]  ", save[i].archive);
-			else
-				fprintf (fp, "arch=[]  ");
-			if (save[i].part)
-				fprintf (fp, "part=[%s]  ", save[i].part);
-			else
-				fprintf (fp, "part=[]  ");
-			if (save[i].patch)
-				fprintf (fp, "patch=[%s]\n", save[i].patch);
-			else
-				fprintf (fp, "patch=[]\n");
-			fprintf (fp,"index=[%d]  saved=[%d]  mailbox=[%d]\n\n", save[i].index, save[i].saved, save[i].is_mailbox);
+			fprintf (fp,"path=[%s]\n", save[i].path);
+			fprintf (fp,"file=[%s]\n", save[i].file);
+			fprintf (fp,"art=[%p]  saved=[%d]  mailbox=[%d]\n\n",
+				save[i].artptr, save[i].saved, save[i].is_mailbox);
 		}
 		fflush (fp);
 		fclose (fp);
@@ -193,29 +206,6 @@ debug_save_comp (
 }
 
 
-static void
-debug_print_base (
-	void)
-{
-	char file[PATH_LEN];
-	FILE *fp;
-	int i;
-
-	if (debug != 2)
-		return;
-
-	sprintf (file, "%sBASE", TMPDIR);
-
-	if ((fp = fopen (file, "a+")) != (FILE *) 0) {
-		for (i = 0; i < top_base; i++)
-			fprintf (fp, "base[%3d]=[%5ld]\n", i, base[i]);
-		fflush (fp);
-		fclose (fp);
-		chmod (file, (S_IRUGO|S_IWUGO));
-	}
-}
-
-
 void
 debug_print_active (
 	void)
@@ -327,7 +317,7 @@ debug_print_filter (
 		the_filter->icase, bool_unparse(bool_not(the_filter->icase)),
 		the_filter->lines_cmp, the_filter->lines_num);
 	fprintf (fp, "       subj=[%s] from=[%s] msgid=[%s]\n",
-		(the_filter->subj  ? the_filter->subj  : ""),
+		BlankIfNull(the_filter->subj),
 		(the_filter->from  ? the_filter->from  : ""),
 		(the_filter->msgid ? the_filter->msgid : ""));
 
@@ -358,7 +348,7 @@ debug_print_filters (
 		num = glob_filter.num;
 		the_filter = glob_filter.filter;
 		fprintf (fp, "*** BEG GLOBAL FILTER=[%3d] ***\n", num);
-		for (i=0 ; i < num ; i++) {
+		for (i = 0; i < num; i++) {
 			debug_print_filter (fp, i, &the_filter[i]);
 			fprintf (fp, "\n");
 		}
@@ -397,7 +387,7 @@ debug_print_active_hash (
 			empty++;
 		} else {
 			number = 1;
-			for (j=group_hash[i]; active[j].next >= 0; j=active[j].next)
+			for (j = group_hash[i]; active[j].next >= 0; j = active[j].next)
 				number++;
 			if (number > 31)
 				my_printf ("MEGA HASH COLLISION > 31 HASH[%d]=[%d]!!!\n", i, number);
@@ -417,16 +407,6 @@ debug_print_active_hash (
 	my_printf ("\n");
 }
 #endif /* 0 */
-
-static void
-debug_print_group_hash (
-	void)
-{
-	int i;
-
-	for (i = 0; i < TABLE_SIZE; i++)
-		my_printf ("group_hash[%4d]=[%4d]\n", i, group_hash[i]);
-}
 #endif /* DEBUG */
 
 #ifdef DEBUG_NEWSRC
@@ -499,7 +479,7 @@ debug_print_newsrc (
 
 	fprintf (fp, "bitmap=[");
 	if (NewSrc->xbitlen && NewSrc->xbitmap) {
-		for (j=0, i=NewSrc->xmin; i <= NewSrc->xmax; i++) {
+		for (j = 0, i = NewSrc->xmin; i <= NewSrc->xmax; i++) {
 			fprintf (fp, "%d",
 				(NTEST(NewSrc->xbitmap, i - NewSrc->xmin) == ART_READ ?
 				ART_READ : ART_UNREAD));
