@@ -5,15 +5,39 @@
  *  Created   : 1992-12-28
  *  Updated   : 1998-02-04
  *  Notes     : Filter articles. Kill & auto selection are supported.
- *  Copyright : (c) Copyright 1991-99 by Iain Lea
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
  *
- *              update for scoring (c) 1997 by Oliver B. Warzecha
+ * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Iain Lea.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
@@ -34,9 +58,12 @@
  * filtering is now done this way:
  * a. set score for all articles and rules
  * b. check each article if the score is above or below the limit
+ *
+ * SET_FILTER is now somewhat shorter, as the real filtering is done
+ * at the end of filter_articles()
  */
 
-#define	SET_FILTER(grp, i, j)	\
+#define SET_FILTER(grp, i, j)	\
 	if (ptr[j].score > 0) { \
 		arts[i].score = (SCORE_MAX - ptr[j].score >= arts[i].score) ? \
 		(arts[i].score + ptr[j].score) : SCORE_MAX ; \
@@ -45,36 +72,34 @@
 		(arts[i].score + ptr[j].score) : -SCORE_MAX ; }
 
 /*
- * SET_FILTER is now somewhat shorter, as the real filtering is done
- * at the end of filter_articles()
+ * These will probably go away when filtering is rewritten
+ * Easier access to hashed msgids. Note that in REFS(), y must be free()d
+ * msgid is mandatory in an article and cannot be NULL
  */
+#define MSGID(x)			(x->refptr->txt)
+#define REFS(x,y)			((y = get_references(x->refptr->parent)) ? y : "")
 
 /*
  * global filter array
  */
 struct t_filters glob_filter = { 0, 0, (struct t_filter *) 0 };
 
-#ifndef INDEX_DAEMON
-	static int *arr_max;
-	static int *arr_num;
-	static struct t_filter *arr_ptr;
-#endif /* !INDEX_DAEMON */
+static int *arr_max;
+static int *arr_num;
+static struct t_filter *arr_ptr;
 
 /*
  * Local prototypes
  */
 static int get_choice (int x, const char *help, const char *prompt, const char *opt1, const char *opt2, const char *opt3, const char *opt4, const char *opt5);
-static int unfilter_articles (void);
 static int set_filter_scope (struct t_group *group);
 static struct t_filter * psExpandFilterArray (struct t_filter *ptr, int *num);
 static t_bool bAddFilterRule (struct t_group *psGrp, struct t_article *psArt, struct t_filter_rule *psRule);
 static void free_filter_array (struct t_filters *ptr);
 static void free_filter_item (struct t_filter *ptr);
-#ifndef INDEX_DAEMON
-	static void vSetFilter (struct t_filter *psFilter);
-	static void vWriteFilterArray (FILE *fp, struct t_filters *ptr, time_t theTime);
-	static void vWriteFilterFile (char *pcFile);
-#endif /* !INDEX_DAEMON */
+static void vSetFilter (struct t_filter *psFilter);
+static void write_filter_array (FILE *fp, struct t_filters *ptr, time_t theTime);
+static void write_filter_file (const char *filename);
 
 
 static struct t_filter *
@@ -110,29 +135,20 @@ test_regex(
 	t_bool nocase,
 	struct regex_cache *cache)
 {
-	const char *regex_errmsg = 0;
 	int regex_errpos;
 
 	if (!tinrc.wildcard) {
 		if (wildmat (string, regex, nocase))
 			return TRUE;
 	} else {
-		if (!cache->re) {
-			if ((cache->re = pcre_compile(regex, PCRE_EXTENDED | ((nocase) ? PCRE_CASELESS : 0),
-												&regex_errmsg, &regex_errpos, NULL)) == NULL)
-				sprintf (mesg, txt_pcre_error_at, regex_errmsg, regex_errpos);
-			else {
-				cache->extra = pcre_study(cache->re, 0, &regex_errmsg);
-				if (regex_errmsg != NULL)
-					sprintf (mesg, txt_pcre_error_text, regex_errmsg);
-			}
-		}
+		if (!cache->re)
+			compile_regex (regex, cache, ((nocase) ? PCRE_CASELESS : 0));
 		if (cache->re) {
 			regex_errpos = pcre_exec(cache->re, cache->extra, string, strlen(string), 0, 0, NULL, 0);
 			if (regex_errpos >= 0)
 				return TRUE;
 			else if (regex_errpos != PCRE_ERROR_NOMATCH)
-				sprintf (mesg, txt_pcre_error_num, regex_errpos);
+				sprintf (mesg, _(txt_pcre_error_num), regex_errpos);
 		}
 	}
 
@@ -143,7 +159,6 @@ test_regex(
 /*
  * vSetFilter() initialises a struct t_filter with default values
  */
-#ifndef INDEX_DAEMON
 static void
 vSetFilter (
 	struct t_filter *psFilter)
@@ -169,7 +184,6 @@ vSetFilter (
 		psFilter->next = (struct t_filter *) 0;
 	}
 }
-#endif /* !INDEX_DAEMON */
 
 
 /*
@@ -212,7 +226,8 @@ free_filter_array (
 
 
 void
-free_all_filter_arrays (void) /* FIXME: use free_filter_array() instead */
+free_all_filter_arrays (
+	void) /* FIXME: use free_filter_array() instead */
 {
 	/*
 	 * Global filter array
@@ -220,14 +235,13 @@ free_all_filter_arrays (void) /* FIXME: use free_filter_array() instead */
 	free_filter_array (&glob_filter);
 }
 
+
 /*
  *  read ~/.tin/filter file contents into filter array
  */
-
-#ifndef INDEX_DAEMON
 t_bool
 read_filter_file (
-	char *file)
+	const char *file)
 {
 	FILE *fp;
 	char *s;
@@ -236,7 +250,7 @@ read_filter_file (
 	char subj[HEADER_LEN];
 	char from[HEADER_LEN];
 	char msgid[HEADER_LEN];
-	char lines[HEADER_LEN];
+	char buffer[HEADER_LEN];
 	char gnksa[HEADER_LEN];
 	char xref[HEADER_LEN];
 	char xref_score[HEADER_LEN];
@@ -251,14 +265,13 @@ read_filter_file (
 	struct t_group *psGrp;
 	t_bool expired = FALSE;
 	t_bool expired_time = FALSE;
-	t_bool global = TRUE;
 	time_t current_secs = (time_t) 0;
 
 	if ((fp = fopen (file, "r")) == (FILE *) 0)
 		return FALSE;
 
-	if (INTERACTIVE)
-		wait_message (0, txt_reading_filter_file);
+	if (!batch_mode)
+		wait_message (0, _(txt_reading_filter_file));
 
 	(void) time (&current_secs);
 
@@ -316,7 +329,7 @@ if (debug) {
 				subj[0] = '\0';
 				from[0] = '\0';
 				msgid[0] = '\0';
-				lines[0] = '\0';
+				buffer[0] = '\0';
 				xref[0] = '\0';
 				icase = 0;
 				secs = 0L;
@@ -341,17 +354,17 @@ if (debug) {
 		break;
 
 		case 'l':
-			if (match_string (buf+1, "ines=", lines, sizeof (lines))) {
+			if (match_string (buf+1, "ines=", buffer, sizeof (buffer))) {
 				if (arr_ptr && !expired_time) {
-					if (lines[0] == '<') {
+					if (buffer[0] == '<') {
 						arr_ptr[i].lines_cmp = FILTER_LINES_LT;
-						arr_ptr[i].lines_num = atoi (&lines[1]);
-					} else if (lines[0] == '>') {
+						arr_ptr[i].lines_num = atoi (&buffer[1]);
+					} else if (buffer[0] == '>') {
 						arr_ptr[i].lines_cmp = FILTER_LINES_GT;
-						arr_ptr[i].lines_num = atoi (&lines[1]);
+						arr_ptr[i].lines_num = atoi (&buffer[1]);
 					} else {
 						arr_ptr[i].lines_cmp = FILTER_LINES_EQ;
-						arr_ptr[i].lines_num = atoi (lines);
+						arr_ptr[i].lines_num = atoi (buffer);
 					}
 				}
 				break;
@@ -400,7 +413,6 @@ if (debug) {
 	my_flush ();
 }
 #endif /* DEBUG */
-				global = TRUE;
 				arr_num = &glob_filter.num;
 				arr_max = &glob_filter.max;
 				if (*arr_num >= (*arr_max - 1))
@@ -414,7 +426,7 @@ if (debug) {
 				subj[0] = '\0';
 				from[0] = '\0';
 				msgid[0] = '\0';
-				lines[0] = '\0';
+				buffer[0] = '\0';
 				xref[0] = '\0';
 				icase = 0;
 				secs = 0L;
@@ -427,11 +439,9 @@ if (debug) {
 
 #ifdef DEBUG
 if (debug) {
-	if (global) {
-		my_printf ("6. buf=[%s]  Gsubj=[%s]\n", arr_ptr[i].subj, glob_filter.filter[i].subj);
-		my_flush ();
-	}
-} /* FIXME: Only global filters now */
+	my_printf ("6. buf=[%s]  Gsubj=[%s]\n", arr_ptr[i].subj, glob_filter.filter[i].subj);
+	my_flush ();
+}
 #endif /* DEBUG */
 				break;
 			}
@@ -537,12 +547,12 @@ if (debug) {
 	fclose (fp);
 
 	if (expired)
-		vWriteFilterFile (file);
+		write_filter_file (file);
 
 	if (cmd_line)
 		printf ("\r\n");
 
-	if (INTERACTIVE)
+	if (!batch_mode)
 		clear_message();
 
 	return TRUE;
@@ -553,33 +563,47 @@ if (debug) {
  */
 
 static void
-vWriteFilterFile (
-	char *pcFile)
+write_filter_file (
+	const char *filename)
 {
-	FILE *hFp;
+	FILE *fp;
+	char *file_tmp;
 
 	if (no_write)
 		return;
 
-	if ((hFp = fopen (pcFile, "w")) == (FILE *) 0)
+	/* generate tmp-filename */
+	file_tmp = get_tmpfilename(filename);
+
+	if (!backup_file (filename, file_tmp))
+	{
+		error_message (_(txt_filesystem_full_backup), filename);
+		free (file_tmp);
+		return;
+	}
+
+	if ((fp = fopen (filename, "w")) == (FILE *) 0)
 		return;
 
-	fprintf (hFp, txt_filter_file, tinrc.filter_days);
-	fflush (hFp);
+	fprintf (fp, _(txt_filter_file));
+	fflush (fp);
 
 	/*
 	 * Save global filters
 	 */
-	vWriteFilterArray (hFp, &glob_filter, time(NULL));
+	write_filter_array (fp, &glob_filter, time(NULL));
 
-	fclose (hFp);
-	chmod (pcFile, (mode_t)(S_IRUSR|S_IWUSR));
+	if (ferror(fp) || fclose (fp)) {
+		error_message (_(txt_filesystem_full), filename);
+		rename (file_tmp, filename);
+	} else
+		unlink (file_tmp);
+	chmod (filename, (mode_t)(S_IRUSR|S_IWUSR));
+	free (file_tmp);
 }
-#endif/* !INDEX_DAEMON */
 
-#ifndef INDEX_DAEMON
 static void
-vWriteFilterArray (
+write_filter_array (
 	FILE *fp,
 	struct t_filters *ptr,
 	time_t theTime)
@@ -592,7 +616,7 @@ vWriteFilterArray (
 
 	for (i = 0; i < ptr->num; i++) {
 
-/* my_printf ("WRITE i=[%d] subj=[%s] from=[%s]\n", i, (ptr->filter[i].subj ? ptr->filter[i].subj : ""), (ptr->filter[i].from ? ptr->filter[i].from : "")); */
+/* my_printf ("WRITE i=[%d] subj=[%s] from=[%s]\n", i, BlankIfNull(ptr->filter[i].subj), BlankIfNull(ptr->filter[i].from)); */
 
 		if (theTime && ptr->filter[i].time) {
 			if (theTime > ptr->filter[i].time)
@@ -684,18 +708,21 @@ my_flush ();
 		if (ptr->filter[i].xref_max > 0) {
 			fprintf (fp, "xref_max=%d\n", ptr->filter[i].xref_max);
 
-			for(j=0;j<ptr->filter[i].xref_score_cnt;j++)
+			for(j = 0; j<ptr->filter[i].xref_score_cnt; j++)
 				fprintf (fp, "xref_score=%d%s%s\n", ptr->filter[i].xref_scores[j], ptr->filter[i].xref_score_strings[j] ? "," : "", ptr->filter[i].xref_score_strings[j]);
 		}
-		if (ptr->filter[i].time)
-			fprintf (fp, "time=%lu\n", (unsigned long int) ptr->filter[i].time);
+		if (ptr->filter[i].time) {
+			char timestring[LEN];
 
+			timestring[0] = '\0';
+			my_strftime(timestring, LEN-1, "%Y-%m-%d %H:%M:%S UTC", gmtime(&(ptr->filter[i].time)));
+			fprintf (fp, "time=%lu (%s)\n", (unsigned long int) ptr->filter[i].time, timestring);
+		}
 		fprintf (fp, "#####\n"); /* makes filter file more readable */
 	}
 
 	fflush (fp);
 }
-#endif /* !INDEX_DAEMON */
 
 
 static int
@@ -744,11 +771,11 @@ get_choice (
 	return (i);
 }
 
+
 /*
  * Interactive filter menu so that the user can dynamically enter parameters.
  * Can be configured for kill or auto-selection screens.
  */
-
 t_bool
 filter_menu (
 	int type,
@@ -769,6 +796,7 @@ filter_menu (
 	char *ptr;
 	char argv[4][PATH_LEN];
 	char buf[LEN];
+	char keyedit[MAXKEYLEN], keyquit[MAXKEYLEN], keysave[MAXKEYLEN];
 	char text_from[PATH_LEN];
 	char text_msgid[PATH_LEN];
 	char text_subj[PATH_LEN];
@@ -799,35 +827,39 @@ filter_menu (
 	 * setup correct text for user selected menu
 	 */
 
+	(void) printascii (keyedit, map_to_local (iKeyFilterEdit, &menukeymap.filter_quit_edit_save));
+	(void) printascii (keyquit, map_to_local (iKeyQuit, &menukeymap.filter_quit_edit_save));
+	(void) printascii (keysave, map_to_local (iKeyFilterSave, &menukeymap.filter_quit_edit_save));
+
 	if (type == FILTER_KILL) {
-		ptr_filter_from = txt_kill_from;
-		ptr_filter_lines = txt_kill_lines;
-		ptr_filter_menu = txt_kill_menu;
-		ptr_filter_msgid = txt_kill_msgid;
-		ptr_filter_scope = txt_kill_scope;
-		ptr_filter_subj = txt_kill_subj;
-		ptr_filter_text = txt_kill_text;
-		ptr_filter_time = txt_kill_time;
-		ptr_filter_help_scope = txt_help_kill_scope;
-		ptr_filter_quit_edit_save = txt_quit_edit_save_kill;
+		ptr_filter_from = _(txt_kill_from);
+		ptr_filter_lines = _(txt_kill_lines);
+		ptr_filter_menu = _(txt_kill_menu);
+		ptr_filter_msgid = _(txt_kill_msgid);
+		ptr_filter_scope = _(txt_kill_scope);
+		ptr_filter_subj = _(txt_kill_subj);
+		ptr_filter_text = _(txt_kill_text);
+		ptr_filter_time = _(txt_kill_time);
+		ptr_filter_help_scope = _(txt_help_kill_scope);
+		ptr_filter_quit_edit_save = _(txt_quit_edit_save_kill);
 	} else {
-		ptr_filter_from = txt_select_from;
-		ptr_filter_lines = txt_select_lines;
-		ptr_filter_menu = txt_select_menu;
-		ptr_filter_msgid = txt_select_msgid;
-		ptr_filter_scope = txt_select_scope;
-		ptr_filter_subj = txt_select_subj;
-		ptr_filter_text = txt_select_text;
-		ptr_filter_time = txt_select_time;
-		ptr_filter_help_scope = txt_help_select_scope;
-		ptr_filter_quit_edit_save = txt_quit_edit_save_select;
+		ptr_filter_from = _(txt_select_from);
+		ptr_filter_lines = _(txt_select_lines);
+		ptr_filter_menu = _(txt_select_menu);
+		ptr_filter_msgid = _(txt_select_msgid);
+		ptr_filter_scope = _(txt_select_scope);
+		ptr_filter_subj = _(txt_select_subj);
+		ptr_filter_text = _(txt_select_text);
+		ptr_filter_time = _(txt_select_time);
+		ptr_filter_help_scope = _(txt_help_select_scope);
+		ptr_filter_quit_edit_save = _(txt_quit_edit_save_select);
 	}
 
-	ptr_filter_score = txt_filter_score;
+	ptr_filter_score = _(txt_filter_score);
 
 	len = cCOLS - 30;
 
-	sprintf (text_time, txt_time_default_days, tinrc.filter_days);
+	sprintf (text_time, _(txt_time_default_days), tinrc.filter_days);
 	sprintf (text_subj, ptr_filter_subj, len, len, art->subject);
 
 	strcpy (buf, art->from);
@@ -851,19 +883,19 @@ filter_menu (
 	my_printf ("%s%s", ptr_filter_scope, group->name);
 	my_flush ();
 
-	show_menu_help (txt_help_filter_text);
+	show_menu_help (_(txt_help_filter_text));
 
 	if (!prompt_menu_string (INDEX_TOP, (int) strlen (ptr_filter_text), rule.text))
 		return FALSE;
 
 	if (*rule.text) {
-		i = get_choice (INDEX_TOP+1, txt_help_filter_text_type,
-			       txt_filter_text_type,
-			       txt_subj_line_only_case,
-			       txt_subj_line_only,
-			       txt_from_line_only_case,
-			       txt_from_line_only,
-			       txt_msgid_line_only);
+		i = get_choice (INDEX_TOP+1, _(txt_help_filter_text_type),
+			       _(txt_filter_text_type),
+			       _(txt_subj_line_only_case),
+			       _(txt_subj_line_only),
+			       _(txt_from_line_only_case),
+			       _(txt_from_line_only),
+			       _(txt_msgid_line_only));
 		if (i == -1)
 			return FALSE;
 
@@ -894,7 +926,7 @@ filter_menu (
 		/*
 		 * Subject:
 		 */
-		i = get_choice (INDEX_TOP+3, txt_help_filter_subj, text_subj, txt_yes, txt_no, (char *)0, (char *)0, (char *)0);
+		i = get_choice (INDEX_TOP+3, _(txt_help_filter_subj), text_subj, _(txt_yes), _(txt_no), (char *)0, (char *)0, (char *)0);
 
 		if (i == -1)
 			return FALSE;
@@ -904,7 +936,7 @@ filter_menu (
 		/*
 		 * From:
 		 */
-		i = get_choice (INDEX_TOP+4, txt_help_filter_from, text_from, (rule.subj_ok ? txt_no : txt_yes), (rule.subj_ok ? txt_yes : txt_no), (char *)0, (char *)0, (char *)0);
+		i = get_choice (INDEX_TOP+4, _(txt_help_filter_from), text_from, (rule.subj_ok ? _(txt_no) : _(txt_yes)), (rule.subj_ok ? _(txt_yes) : _(txt_no)), (char *)0, (char *)0, (char *)0);
 
 		if (i == -1)
 			return FALSE;
@@ -915,9 +947,9 @@ filter_menu (
 		 * Message-Id:
 		 */
 		if (rule.subj_ok || rule.from_ok)
-			i = get_choice (INDEX_TOP+5, txt_help_filter_msgid, text_msgid, txt_no, txt_full, txt_last, txt_only, (char *)0);
+			i = get_choice (INDEX_TOP+5, _(txt_help_filter_msgid), text_msgid, _(txt_no), _(txt_full), _(txt_last), _(txt_only), (char *)0);
 		else
-			i = get_choice (INDEX_TOP+5, txt_help_filter_msgid, text_msgid, txt_full, txt_last, txt_only, txt_no, (char *)0);
+			i = get_choice (INDEX_TOP+5, _(txt_help_filter_msgid), text_msgid, _(txt_full), _(txt_last), _(txt_only), _(txt_no), (char *)0);
 
 		if (i == -1)
 			return FALSE;
@@ -956,7 +988,7 @@ filter_menu (
 	/*
 	 * Lines:
 	 */
-	show_menu_help (txt_help_filter_lines);
+	show_menu_help (_(txt_help_filter_lines));
 
 	buf[0] = '\0';
 
@@ -989,7 +1021,7 @@ filter_menu (
 	 * Scoring value
 	 */
 	buf[0] = '\0';
-	show_menu_help(txt_filter_score_help); /* FIXME: a sprintf() is necessary here */
+	show_menu_help(_(txt_filter_score_help)); /* FIXME: a sprintf() is necessary here */
 
 	if (!prompt_menu_string(INDEX_TOP+8, (int)strlen(ptr_filter_score), buf))
 		return FALSE;
@@ -1011,7 +1043,8 @@ filter_menu (
 	 */
 	sprintf (double_time, "2x %s", text_time);
 	sprintf (quat_time, "4x %s", text_time);
-	i = get_choice (INDEX_TOP+9, txt_help_filter_time, ptr_filter_time, txt_unlimited_time, text_time, double_time, quat_time, (char *)0);
+	i = get_choice (INDEX_TOP+9, _(txt_help_filter_time), ptr_filter_time,
+			_(txt_unlimited_time), text_time, double_time, quat_time, (char *)0);
 
 	if (i == -1)
 		return FALSE;
@@ -1023,7 +1056,7 @@ filter_menu (
 	 */
 	if (*rule.text || rule.subj_ok || rule.from_ok || rule.msgid_ok || rule.lines_ok) {
 		strcpy (argv[0], group->name);
-		strcpy (argv[1], txt_all_groups);
+		strcpy (argv[1], _(txt_all_groups));
 		strcpy (argv[2], group->name);
 		ptr = strrchr (argv[2], '.');
 		if (ptr != (char *) 0) {
@@ -1059,17 +1092,18 @@ filter_menu (
 		return FALSE;
 
 	forever {
-		ch = prompt_slk_response(ch_default, "eqs\033", ptr_filter_quit_edit_save);
+		ch = prompt_slk_response (ch_default,
+				&menukeymap.filter_quit_edit_save,
+				ptr_filter_quit_edit_save,
+				keyquit, keyedit, keysave);
 		switch (ch) {
 
 		case iKeyFilterEdit:
 			bAddFilterRule (group, art, &rule); /* save the rule */
-			if (!invoke_editor (filter_file, 25)) /* FIXME: is 25 correct offset? */
+			if (!invoke_editor (filter_file, 25)) /* FIXME: is 25 correct offset ? */
 				return FALSE;
 			unfilter_articles ();
-#ifndef INDEX_DAEMON
 			(void) read_filter_file (filter_file);
-#endif /* !INDEX_DAEMON */
 			return TRUE;
 			/* keep lint quiet: */
 			/* FALLTHROUGH */
@@ -1163,7 +1197,7 @@ quick_filter (
 t_bool
 quick_filter_select_posted_art (
 	struct t_group *group,
-	char *subj)	/* return value is always ignored */
+	const char *subj)	/* return value is always ignored */
 {
 	t_bool filtered = FALSE;
 
@@ -1264,13 +1298,13 @@ bAddFilterRule (
 	switch(psRule->expire_time)
 	{
 		case 1:
-			psPtr[*plNum].time = lCurTime + (time_t) (tinrc.filter_days * 86400);	/*  86400 = 60 * 60 * 24 */
+			psPtr[*plNum].time = lCurTime + (time_t) (tinrc.filter_days * DAY);
 			break;
 		case 2:
-			psPtr[*plNum].time = lCurTime + (time_t) (tinrc.filter_days * 172800);	/* 172800 = 60 * 60 * 24 * 2 */
+			psPtr[*plNum].time = lCurTime + (time_t) (tinrc.filter_days * DAY * 2);
 			break;
 		case 3:
-			psPtr[*plNum].time = lCurTime + (time_t) (tinrc.filter_days * 345600);	/* 345600 = 60 * 60 * 24 * 4 */
+			psPtr[*plNum].time = lCurTime + (time_t) (tinrc.filter_days * DAY * 4);
 			break;
 		default:
 			psPtr[*plNum].time = (time_t) 0;
@@ -1342,18 +1376,16 @@ bAddFilterRule (
 		if (debug)
 			wait_message (2, "inscope=[%s] scope=[%s] typ=[%d] case=[%d] subj=[%s] from=[%s] msgid=[%s] fullref=[%d] line=[%d %d] time=[%lu]",
 				bool_unparse(psPtr[*plNum-1].inscope),
-				(psRule->scope ? psRule->scope : ""),
+				BlankIfNull(psRule->scope),
 				psPtr[*plNum-1].type, psPtr[*plNum-1].icase,
-				(psPtr[*plNum-1].subj ? psPtr[*plNum-1].subj : ""),
-				(psPtr[*plNum-1].from ? psPtr[*plNum-1].from : ""),
-				(psPtr[*plNum-1].msgid ? psPtr[*plNum-1].msgid : ""),
+				BlankIfNull(psPtr[*plNum-1].subj),
+				BlankIfNull(psPtr[*plNum-1].from),
+				BlankIfNull(psPtr[*plNum-1].msgid),
 				psPtr[*plNum-1].fullref, psPtr[*plNum-1].lines_cmp,
 				psPtr[*plNum-1].lines_num, (unsigned long int) psPtr[*plNum-1].time);
 #endif /* DEBUG */
 
-#ifndef INDEX_DAEMON
-		vWriteFilterFile (filter_file);
-#endif /* !INDEX_DAEMON */
+		write_filter_file (filter_file);
 	}
 
 	return bFiltered;
@@ -1365,13 +1397,14 @@ bAddFilterRule (
  * tagged as being read BECAUSE they were killed. So, we retag
  * them as being unread.
  */
-static int
-unfilter_articles (void) /* return value is always ignored */
+int
+unfilter_articles (
+	void) /* return value is always ignored */
 {
 	int unkilled = 0;
 	register int i;
 
-	for (i = 0; i < top; i++) {
+	for (i = 0; i < top_art; i++) {
 		if (IS_KILLED(i)) {
 			arts[i].killed = FALSE;
 			arts[i].status = ART_UNREAD;
@@ -1423,8 +1456,8 @@ filter_articles (
 	 * ie. group=comp.os.linux.help  scope=comp.os.linux.*
 	 */
 	inscope = set_filter_scope (group);
-	if (!batch_mode /* !cmd_line */)
-		wait_message (0, txt_filter_global_rules, inscope, group->glob_filter->num);
+	if (!cmd_line && !batch_mode)
+		wait_message (0, _(txt_filter_global_rules), inscope, group->glob_filter->num);
 	num = group->glob_filter->num;
 	ptr = group->glob_filter->filter;
 
@@ -1455,14 +1488,14 @@ filter_articles (
 	/*
 	 * loop thru all arts applying global & local filtering rules
 	 */
-	for (i = 0; i < top; i++) {
+	for (i = 0; i < top_art; i++) {
 		arts[i].score = 0;
 
+		mesg[0] = '\0';				/* Clear system message field */
+
 		if (tinrc.kill_level == KILL_READ && IS_READ(i)) /* skip only when the article is read */
 			continue;
 
-		mesg[0] = '\0';				/* Clear system message field */
-
 		for (j = 0; j < num; j++) {
 			if (ptr[j].inscope) {
 				/*
@@ -1541,28 +1574,28 @@ filter_articles (
 				/*
 				 * Filter on Lines: line
 				 */
-				if ((ptr[j].lines_cmp != FILTER_LINES_NO) && (arts[i].lines >= 0)) {
+				if ((ptr[j].lines_cmp != FILTER_LINES_NO) && (arts[i].line_count >= 0)) {
 					switch (ptr[j].lines_cmp) {
 						case FILTER_LINES_EQ:
-							if (arts[i].lines == ptr[j].lines_num) {
+							if (arts[i].line_count == ptr[j].lines_num) {
 /*
-wait_message (1, "FILTERED Lines arts[%d] == [%d]", arts[i].lines, ptr[j].lines_num);
+wait_message (1, "FILTERED Lines arts[%d] == [%d]", arts[i].line_count, ptr[j].lines_num);
 */
 								SET_FILTER(group, i, j);
 							}
 							break;
 						case FILTER_LINES_LT:
-							if (arts[i].lines < ptr[j].lines_num) {
+							if (arts[i].line_count < ptr[j].lines_num) {
 /*
-wait_message (1, "FILTERED Lines arts[%d] < [%d]", arts[i].lines, ptr[j].lines_num);
+wait_message (1, "FILTERED Lines arts[%d] < [%d]", arts[i].line_count, ptr[j].lines_num);
 */
 								SET_FILTER(group, i, j);
 							}
 							break;
 						case FILTER_LINES_GT:
-							if (arts[i].lines > ptr[j].lines_num) {
+							if (arts[i].line_count > ptr[j].lines_num) {
 /*
-wait_message (1, "FILTERED Lines arts[%d] > [%d]", arts[i].lines, ptr[j].lines_num);
+wait_message (1, "FILTERED Lines arts[%d] > [%d]", arts[i].line_count, ptr[j].lines_num);
 */
 								SET_FILTER(group, i, j);
 							}
@@ -1612,7 +1645,7 @@ wait_message (1, "FILTERED Lines arts[%d] > [%d]", arts[i].lines, ptr[j].lines_n
 						while(*s && isspace((int)*s))
 							s++;
 
-						group_count=0;
+						group_count = 0;
 
 						while(*s) {
 							e = s;
@@ -1620,8 +1653,8 @@ wait_message (1, "FILTERED Lines arts[%d] > [%d]", arts[i].lines, ptr[j].lines_n
 								e++;
 
 							if (ptr[j].xref_max > 0) {
-								strncpy(buf, s, e-s);
-								buf[e-s] = '\0';
+								strncpy(buf, s, e - s);
+								buf[e - s] = '\0';
 								for (k = 0; k < ptr[j].xref_score_cnt; k++) {
 									if (GROUP_MATCH(buf, ptr[j].xref_score_strings[k], TRUE)) {
 										group_count += ptr[j].xref_scores[k];
@@ -1632,8 +1665,8 @@ wait_message (1, "FILTERED Lines arts[%d] > [%d]", arts[i].lines, ptr[j].lines_n
 									group_count++;
 							}
 							if (ptr[j].xref != (char*)0) {
-								strncpy(buf, s, e-s);
-								buf[e-s] = '\0';
+								strncpy(buf, s, e - s);
+								buf[e - s] = '\0';
 								/* don't filter when we are actually in that group */
 								/* Group names shouldn't be case sensitive in any case. Whatever */
 								if (ptr[j].type != FILTER_KILL || strcmp(group->name, buf) != 0) {
@@ -1641,7 +1674,7 @@ wait_message (1, "FILTERED Lines arts[%d] > [%d]", arts[i].lines, ptr[j].lines_n
 										group_count = -1;
 								}
 							}
-							s=e;
+							s = e;
 							while(*s && !isspace((int)*s))
 								s++;
 							while(*s && isspace((int)*s))
@@ -1684,7 +1717,7 @@ wait_message (1, "FILTERED Lines arts[%d] > [%d]", arts[i].lines, ptr[j].lines_n
 	 * all articles have scored, so do kill & select
 	 */
 
-	for (i = 0; i < top; i++) {
+	for (i = 0; i < top_art; i++) {
 		if (arts[i].score <= SCORE_LIM_KILL) {
 			arts[i].killed = TRUE;
 			num_of_killed_arts++;
