@@ -5,13 +5,39 @@
  *  Created   : 1993-12-18
  *  Updated   : 1997-01-07
  *  Notes     : Low level functions handling the active[] list and its group_hash index
- *  Copyright : (c) Copyright 1991-99 by Iain Lea
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1993-2001 Iain Lea <iain@bricbrac.de>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Iain Lea.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
@@ -46,7 +72,7 @@ hash_groupname (
 
 	for (p = group; *p; p++) {
 		hash = (hash << 4) + *p;
-		if (g = hash & 0xf0000000) {
+		if ((g = hash & 0xf0000000) != 0) {
 			hash ^= g >> 24;
 			hash ^= g;
 		}
@@ -103,12 +129,12 @@ find_group_index (
  *  Find group name in active[] array and return pointer to element
  */
 struct t_group *
-psGrpFind (
-	char *pcGrpName)
+group_find (
+	const char *group_name)
 {
 	int i;
 
-	if ((i = find_group_index(pcGrpName)) != -1)
+	if ((i = find_group_index(group_name)) != -1)
 		return &active[i];
 
 	return (struct t_group *) 0;
@@ -120,8 +146,8 @@ psGrpFind (
  * Return pointer to next free active slot or NULL if duplicate
  */
 struct t_group *
-psGrpAdd (
-	char *group)
+group_add (
+	const char *group)
 {
 	unsigned long h;
 	int i;
