@@ -5,13 +5,39 @@
  *  Created   : 1991-04-01
  *  Updated   : 1997-12-28
  *  Notes     :
- *  Copyright : (c) Copyright 1991-99 by Iain Lea & Rich Skrenta
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Iain Lea, Rich Skrenta
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
@@ -42,17 +68,20 @@ static char **cmdargs;
 static int num_cmdargs;
 static int max_cmdargs;
 
+t_bool catchup = FALSE;			/* mark all arts read in all subscribed groups */
+t_bool check_any_unread = FALSE;/* print/return status if any unread */
+t_bool mail_news = FALSE;		/* mail all arts to specified user */
+t_bool save_news = FALSE;		/* save all arts to savedir structure */
+t_bool start_any_unread = FALSE;/* only start if unread news */
+
+
 /*
  * Local prototypes
  */
-static int check_for_any_new_news (t_bool CheckAnyUnread, t_bool StartAnyUnread);
-static void save_or_mail_new_news (void);
 static void read_cmd_line_options (int argc, char *argv[]);
 static void update_index_files (void);
 static void usage (char *theProgname);
-#ifndef INDEX_DAEMON
-	static void show_intro_page (void);
-#endif /* !INDEX_DAEMON */
+static void show_intro_page (void);
 
 
 /*
@@ -68,6 +97,11 @@ main (
 	int count;
 	t_bool tmp_no_write;
 
+	/* initialize locale support */
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+
 	set_signal_handlers ();
 
 	cmd_line = TRUE;
@@ -99,7 +133,7 @@ main (
 #	ifdef NNTP_ABLE
 		read_news_via_nntp = TRUE;
 #	else
-		error_message (txt_option_not_enabled, "-DNNTP_ABLE");
+		error_message (_(txt_option_not_enabled), "-DNNTP_ABLE");
 		giveup();
 #	endif /* NNTP_ABLE */
 	}
@@ -134,39 +168,36 @@ main (
 	tmp_no_write = no_write; /* keep no_write */
 	no_write = TRUE;		 /* don't allow any writing back during startup */
 
-#if defined(M_UNIX) && !defined(INDEX_DAEMON)
+	if (!batch_mode) {
+#ifdef M_UNIX
 #	ifndef USE_CURSES
-	if (INTERACTIVE) {
 		if (!get_termcaps ()) {
-			error_message (txt_screen_init_failed, tin_progname);
+			error_message (_(txt_screen_init_failed), tin_progname);
 			giveup();
 		}
-/*		EndInverse ();*/
-	}
 #	endif /* !USE_CURSES */
-#endif /* M_UNIX && !INDEX_DAEMON */
+#endif /* M_UNIX */
 
-	/*
-	 * Init curses emulation
-	 */
-	if (!InitScreen ()) {
-		error_message (txt_screen_init_failed, tin_progname);
-		giveup();
+		/*
+		 * Init curses emulation
+		 */
+		if (!InitScreen ()) {
+			error_message (_(txt_screen_init_failed), tin_progname);
+			giveup();
+		}
+
+		EndInverse ();
+
+		/*
+		 * This depends on various things in tinrc
+		 */
+		setup_screen ();
 	}
 
-	EndInverse ();
-
-	/*
-	 * This depends on various things in tinrc
-	 */
-	setup_screen ();
-
-	if (INTERACTIVE || (batch_mode && verbose))
+	if (!batch_mode || (batch_mode && verbose))
 		wait_message (0, "%s\n", cvers);
 
-#ifndef INDEX_DAEMON
 	set_up_private_index_cache ();
-#endif /* !INDEX_DAEMON */
 
 	/*
 	 * Connect to nntp server?
@@ -186,33 +217,36 @@ main (
 /*	vNewsrcTestHarness ();*/
 #endif /* DEBUG_NEWSRC */
 
+	/*
+	 * Read user specific keybindings
+	 */
+	read_keymap_file ();
+
 	/*
 	 * Read input history
 	 */
-#ifndef INDEX_DAEMON
 	if (!batch_mode)
 		read_input_history_file ();
-#endif /* !INDEX_DAEMON */
 
 	/*
 	 * Load the mail & news active files into active[]
 	 *
 	 * create_save_active_file cannot write to active.save
-	 * if no_write == TRUE, so restore original value temporarily
+	 * if no_write != FALSE, so restore original value temporarily
 	 */
 	no_write = tmp_no_write;
 	if (read_saved_news)
 		create_save_active_file ();
 	no_write = TRUE;
 
-#if !defined(INDEX_DAEMON) && defined(HAVE_MH_MAIL_HANDLING)
+#ifdef HAVE_MH_MAIL_HANDLING
 	read_mail_active_file ();
-#endif /* !INDEX_DAEMON && HAVE_MH_MAIL_HANDLING */
+#endif /* HAVE_MH_MAIL_HANDLING */
 
 	/*
 	 * Initialise active[] and add new newsgroups to start of my_group[]
 	 */
-	group_top = 0;
+	selmenu.max = 0;
 	read_news_active_file ();
 #ifdef DEBUG
 	debug_print_active();
@@ -220,26 +254,21 @@ main (
 
 	/*
 	 * Load the local & global group specific attribute files
+	 * FIXME: is global attributes still needed? if so use TIN_DEFAULTS_DIR
 	 */
-#ifndef INDEX_DAEMON
-	if (INTERACTIVE)
-		wait_message (0, txt_reading_attributes_file, "global ");
+	if (!batch_mode /* || (batch_mode && verbose)*/) /* FIXME: NLS support missing */
+		wait_message (0, _(txt_reading_attributes_file), _(txt_global));
 	read_attributes_file (global_attributes_file, TRUE);
-	if (INTERACTIVE)
-		wait_message (0, txt_reading_attributes_file, "");
+	if (!batch_mode /* || (batch_mode && verbose)*/)
+		wait_message (0, _(txt_reading_attributes_file), "");
 	read_attributes_file (local_attributes_file, FALSE);
-#endif /* !INDEX_DAEMON */
 
 	/*
 	 * Read in users filter preferences file.
 	 * This has to be done before quick post
 	 * because the filters will be updated.
 	 */
-#ifndef INDEX_DAEMON
 	filtered_articles = read_filter_file (filter_file);
-#else
-	filtered_articles = TRUE;
-#endif /* !INDEX_DAEMON */
 
 #ifdef DEBUG
 	debug_print_filters ();
@@ -248,29 +277,27 @@ main (
 	/*
 	 * Quick post an article & exit if -w specified
 	 */
-#ifndef INDEX_DAEMON
 	if (post_article_and_exit || post_postponed_and_exit) {
 		quick_post_article (post_postponed_and_exit);
-		wait_message (2, txt_exiting);
+		wait_message (2, _(txt_exiting));
 		tin_done (EXIT_SUCCESS);
 	}
-#endif /* !INDEX_DAEMON */
 
 	if ((count = count_postponed_articles()))
-		wait_message(3, txt_info_postponed, count, IS_PLURAL(count));
+		wait_message(3, _(txt_info_postponed), count, PLURAL(count, txt_article));
 
 	/*
 	 * Read text descriptions for mail and/or news groups
 	 */
-#if !defined(INDEX_DAEMON) && defined(HAVE_MH_MAIL_HANDLING)
-	read_mailgroups_file ();
-#endif /* !INDEX_DAEMON && HAVE_MH_MAIL_HANDLING */
-	read_newsgroups_file ();
+	if (show_description && !batch_mode) {
+#ifdef HAVE_MH_MAIL_HANDLING
+		read_mailgroups_file ();
+#endif /* HAVE_MH_MAIL_HANDLING */
+		read_newsgroups_file ();
+	}
 
-#ifndef INDEX_DAEMON
 	if (create_mail_save_dirs ())
 		write_config_file (local_config_file);
-#endif /* !INDEX_DAEMON */
 
 	/*
 	 * Preloads active[] with command line groups. They will follow any
@@ -278,15 +305,19 @@ main (
 	 */
 	num_cmd_line_groups = read_cmd_line_groups ();
 
-#ifdef INDEX_DAEMON
-	vMakeActiveMyGroup ();
-#else
 	backup_newsrc ();
 
 	/*
 	 * Load my_groups[] from the .newsrc file. We append these groups to any
 	 * new newsgroups and command line newsgroups already loaded
 	 */
+
+	/* TODO:
+	 * if (num_cmd_line_groups != 0 && check_any_unread)
+	 * don't read newsrc.
+	 * This makes -Z handle command line newsgroups. Test & document
+	 */
+
 	read_newsrc_lines = read_newsrc (newsrc, FALSE);
 
 	no_write = tmp_no_write; /* restore old value */
@@ -298,40 +329,59 @@ main (
 		tinrc.show_only_unread_groups = FALSE;
 	else
 		toggle_my_groups (tinrc.show_only_unread_groups, "");
-#endif /* INDEX_DAEMON */
 
 	/*
 	 * Check/start if any new/unread articles
 	 */
-	start_groupnum = check_for_any_new_news (check_any_unread, start_any_unread);
+	if (check_any_unread)
+		exit (check_start_save_any_news (CHECK_ANY_NEWS, catchup));
+
+	if (start_any_unread) {
+		batch_mode = TRUE;			/* Suppress some unwanted on-screen garbage */
+		if ((start_groupnum = check_start_save_any_news (START_ANY_NEWS, catchup)) == -1)
+			giveup();				/* No new/unread news so exit */
+		batch_mode = FALSE;
+	}
 
 	/*
 	 * Mail any new articles to specified user
 	 * or
 	 * Save any new articles to savedir structure for later reading
 	 */
-	save_or_mail_new_news ();
+	if (mail_news || save_news) {
+		do_update (FALSE);
+		check_start_save_any_news (mail_news ? MAIL_ANY_NEWS : SAVE_ANY_NEWS, catchup);
+		tin_done (EXIT_SUCCESS);
+	}
 
 	/*
 	 * Catchup newsrc file (-c option)
 	 */
-	catchup_newsrc_file ();
+	if (catchup) {
+		catchup_newsrc_file ();
+		tin_done (EXIT_SUCCESS);
+	}
 
 	/*
 	 * Update index files
+	 * Only the -u batch_mode case will get this far
 	 */
-	update_index_files ();
+	if (batch_mode)
+		update_index_files ();
 
 	/*
 	 * If first time print welcome screen and auto-subscribe
 	 * to groups specified in /usr/lib/news/subscribe locally
 	 * or via NNTP if reading news remotely (LIST SUBSCRIBE)
 	 */
-#ifndef INDEX_DAEMON
 	if (created_rcdir && !batch_mode)
 		show_intro_page ();
 
-#endif /* !INDEX_DAEMON */
+#ifdef USE_CURSES
+	/* Turn scrolling off now the startup messages have been displayed */
+	scrollok (stdscr, FALSE);
+#endif /* USE_CURSES */
+
 	/*
 	 * Work loop
 	 */
@@ -343,15 +393,11 @@ main (
 /*
  * process command line options
  */
-#ifndef INDEX_DAEMON
-#	ifndef M_AMIGA
-#		define OPTIONS "aAcdD:f:G:g:hHI:lm:M:nNop:qQrRs:SuUvVwXzZ"
-#	else
-#		define OPTIONS "BcdD:f:G:hHI:lm:M:nNop:qQrRs:SuUvVwXzZ"
-#	endif /* M_AMIGA */
+#ifndef M_AMIGA
+#	define OPTIONS "aAcdD:f:G:g:hHI:lm:M:nNop:qQrRs:SuvVwXzZ"
 #else
-#	define OPTIONS "dD:f:G:hI:PvV"
-#endif /* !INDEX_DAEMON */
+#	define OPTIONS "BcdD:f:G:hHI:lm:M:nNop:qQrRs:SuvVwXzZ"
+#endif /* M_AMIGA */
 
 static void
 read_cmd_line_options (
@@ -365,13 +411,12 @@ read_cmd_line_options (
 
 	while ((ch = getopt (argc, argv, OPTIONS)) != -1) {
 		switch (ch) {
-#ifndef INDEX_DAEMON
 #	ifndef M_AMIGA
 			case 'a':
 #		ifdef HAVE_COLOR
 				use_color = !use_color;
 #		else
-				error_message (txt_option_not_enabled, "-DHAVE_COLOR");
+				error_message (_(txt_option_not_enabled), "-DHAVE_COLOR");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -382,7 +427,7 @@ read_cmd_line_options (
 #		ifdef NNTP_ABLE
 				force_auth_on_conn_open = TRUE;
 #		else
-				error_message (txt_option_not_enabled, "-DNNTP_ABLE");
+				error_message (_(txt_option_not_enabled), "-DNNTP_ABLE");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -401,18 +446,13 @@ read_cmd_line_options (
 			case 'd':
 				show_description = FALSE;
 				break;
-#else
-			case 'd':		/* delete index file before indexing */
-				delete_index_file = TRUE;
-				break;
-#endif /* !INDEX_DAEMON */
 
 			case 'D':		/* debug mode 1=NNTP 2=ALL */
 #ifdef DEBUG
 				debug = atoi (optarg);
 				debug_delete_files ();
 #else
-				error_message (txt_option_not_enabled, "-DDEBUG");
+				error_message (_(txt_option_not_enabled), "-DDEBUG");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -420,12 +460,8 @@ read_cmd_line_options (
 				break;
 
 			case 'f':	/* active (tind) / newsrc (tin) file */
-#ifdef INDEX_DAEMON
-				my_strncpy (news_active_file, optarg, sizeof (news_active_file));
-#else
 				my_strncpy (newsrc, optarg, sizeof (newsrc));
 				newsrc_set = TRUE;
-#endif /* INDEX_DAEMON */
 				break;
 
 			case 'G':
@@ -436,14 +472,13 @@ read_cmd_line_options (
 					tinrc.use_getart_limit = FALSE;
 				break;
 
-#ifndef INDEX_DAEMON
 #	ifndef M_AMIGA
 			case 'g':	/* select alternative NNTP-server, implies -r */
 #		ifdef NNTP_ABLE
 				my_strncpy(cmdline_nntpserver, optarg, sizeof(cmdline_nntpserver));
 				read_news_via_nntp = TRUE;
 #		else
-				error_message (txt_option_not_enabled, "-DNNTP_ABLE");
+				error_message (_(txt_option_not_enabled), "-DNNTP_ABLE");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -456,21 +491,19 @@ read_cmd_line_options (
 				exit (EXIT_SUCCESS);
 				/* keep lint quiet: */
 				/* FALLTHROUGH */
-#endif /* !INDEX_DAEMON */
 
 			case 'I':
 #ifndef NNTP_ONLY
 				my_strncpy (index_newsdir, optarg, sizeof (index_newsdir));
-				my_mkdir (index_newsdir, (mode_t)S_IRWXUGO);
+/*				my_mkdir (index_newsdir, (mode_t)S_IRWXUGO);*/
 #else
-				error_message (txt_option_not_enabled, "-DNNTP_ABLE");
+				error_message (_(txt_option_not_enabled), "-DNNTP_ABLE");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
 #endif /* !NNTP_ONLY */
 				break;
 
-#ifndef INDEX_DAEMON
 			case 'l':
 				list_active = TRUE;
 				break;
@@ -498,11 +531,6 @@ read_cmd_line_options (
 			case 'o':	/* post postponed articles & exit */
 				post_postponed_and_exit = TRUE;
 				break;
-#else
-			case 'P':	/* stat every art for a through purge */
-				purge_index_files = TRUE;
-				break;
-#endif /* !INDEX_DAEMON */
 
 #ifdef NNTP_ABLE
 			case 'p': /* implies -r */
@@ -512,7 +540,6 @@ read_cmd_line_options (
 				break;
 #endif /* NNTP_ABLE */
 
-#ifndef INDEX_DAEMON
 			case 'q':
 				check_for_new_newsgroups = FALSE;
 				break;
@@ -527,7 +554,7 @@ read_cmd_line_options (
 #	ifdef NNTP_ABLE
 				read_news_via_nntp = TRUE;
 #	else
-				error_message (txt_option_not_enabled, "-DNNTP_ABLE");
+				error_message (_(txt_option_not_enabled), "-DNNTP_ABLE");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -556,43 +583,157 @@ read_cmd_line_options (
 				batch_mode = TRUE;
 				show_description = FALSE;
 #	else
-				error_message (txt_option_not_enabled, "-DNNTP_ABLE");
+				error_message (_(txt_option_not_enabled), "-DNNTP_ABLE");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
 #	endif /* !NNTP_ONLY */
 				break;
 
-			case 'U':	/* update index files in background */
-#	ifndef NNTP_ONLY
-				update_fork = TRUE;
-				batch_mode = TRUE;
-#	else
-				error_message (txt_option_not_enabled, "-DNNTP_ABLE");
-				giveup();
-				/* keep lint quiet: */
-				/* NOTREACHED */
-#	endif /* !NNTP_ONLY */
-				break;
-
-#endif /* !INDEX_DAEMON */
-
 			case 'v':	/* verbose mode */
 				verbose = TRUE;
 				break;
 
 			case 'V':
 #if defined(__DATE__) && defined(__TIME__)
-				error_message ("Version: %s release %s (\"%s\") %s %s",
+				error_message (_("Version: %s release %s (\"%s\") %s %s"),
 					VERSION, RELEASEDATE, RELEASENAME, __DATE__, __TIME__);
 #else
-				error_message ("Version: %s release %s (\"%s\")",
+				error_message (_("Version: %s release %s (\"%s\")"),
 					VERSION, RELEASEDATE, RELEASENAME);
-#endif /* __DATE__  && __TIME__ */
+#endif /* __DATE__ && __TIME__ */
+/*
+ * FIXME: make the rest below a function, so we can also use it
+ * inside post.c:mail_bug_report()
+ */
+#ifdef SYSTEM_NAME
+				error_message("Platform:");
+				error_message("\tOS-Name  = \"%s\"", SYSTEM_NAME);
+#endif /* SYSTEM_NAME */
+#ifdef TIN_CC
+				error_message("Compiler:");
+				error_message("\tCC       = \"%s\"", TIN_CC);
+#	ifdef TIN_CFLAGS
+				error_message("\tCFLAGS   = \"%s\"", TIN_CFLAGS);
+#	endif /* TIN_CFLAGS */
+#	ifdef	TIN_CPP
+				error_message("\tCPP      = \"%s\"", TIN_CPP);
+#	endif /* TIN_CPP */
+#	ifdef TIN_CPPFLAGS
+				error_message("\tCPPFLAGS = \"%s\"", TIN_CPPFLAGS);
+#	endif /* TIN_CPPFLAGS */
+#endif /* TIN_CC */
+#ifdef TIN_LD
+				error_message("Linker and Libraries:");
+				error_message("\tLD       = \"%s\"", TIN_LD);
+#	ifdef TIN_LDFLAGS
+				error_message("\tLDFLAGS  = \"%s\"", TIN_LDFLAGS);
+#	endif /* TIN_LDFLAGS */
+#	ifdef TIN_LIBS
+				error_message("\tLIBS     = \"%s\"", TIN_LIBS);
+#	endif /* TIN_LIBS */
+#endif /* TIN_LD */
+				error_message("Characteristics:");
+				error_message("\t"
+/* TODO: complete list and do some usefull grouping */
+#ifdef NNTP_ONLY
+				"+NNTP_ONLY "
+#else
+#	ifdef NNTP_ABLE
+				"+NNTP_ABLE "
+#	else
+				"-NNTP_ABLE "
+#	endif /* NNTP_ABLE */
+#endif /* NNTP_ONLY */
+#ifdef NO_POSTING
+				"+NO_POSTING "
+#else
+				"-NO_POSTING "
+#endif /* NO_POSTING */
+#ifdef HAVE_FASCIST_NEWSADMIN
+				"+HAVE_FASCIST_NEWSADMIN "
+#else
+				"-HAVE_FASCIST_NEWSADMIN "
+#endif /* HAVE_FASCIST_NEWSADMIN */
+#ifdef ENABLE_IPV6
+				"+ENABLE_IPV6"
+#else
+				"-ENABLE_IPV6"
+#endif /* ENABLE_IPV6 */
+				"\n\t"
+#ifdef NO_SHELL_ESCAPE
+				"+NO_SHELL_ESCAPE "
+#else
+				"-NO_SHELL_ESCAPE "
+#endif /* NO_SHELL_ESCAPE */
+#ifdef DISABLE_PRINTING
+				"+DISABLE_PRINTING "
+#else
+				"-DISABLE_PRINTING "
+#endif /* DISABLE_PRINTING */
+#ifdef DONT_HAVE_PIPING
+				"+DONT_HAVE_PIPING "
+#else
+				"-DONT_HAVE_PIPING "
+#endif /* DONT_HAVE_PIPING */
+#ifdef NO_ETIQUETTE
+				"+NO_ETIQUETTE"
+#else
+				"-NO_ETIQUETTE"
+#endif /* NO_ETIQUETTE */
+				"\n\t"
+#ifdef HAVE_LONG_FILE_NAMES
+				"+HAVE_LONG_FILE_NAMES "
+#else
+				"-HAVE_LONG_FILE_NAMES "
+#endif /* HAVE_LONG_FILE_NAMES */
+#ifdef APPEND_PID
+				"+APPEND_PID"
+#else
+				"-APPEND_PID"
+#endif /* APPEND_PID */
+				"\n\t"
+#ifdef HAVE_COLOR
+				"+HAVE_COLOR "
+#else
+				"-HAVE_COLOR "
+#endif /* HAVE_COLOR */
+#ifdef HAVE_PGP
+				"+HAVE_PGP "
+#else
+				"-HAVE_PGP "
+#endif /* HAVE_PGP */
+#ifdef HAVE_PGPK
+				"+HAVE_PGPK "
+#else
+				"-HAVE_PGPK "
+#endif /* HAVE_PGPK */
+#ifdef HAVE_GPG
+				"+HAVE_GPG "
+#else
+				"-HAVE_GPG "
+#endif /* HAVE_GPG */
+				"\n\t"
+#ifdef MIME_BREAK_LONG_LINES
+				"+MIME_BREAK_LONG_LINES "
+#else
+				"-MIME_BREAK_LONG_LINES "
+#endif /* MIME_BREAK_LONG_LINES */
+#ifdef MIME_STRICT_CHARSET
+				"+MIME_STRICT_CHARSET "
+#else
+				"-MIME_STRICT_CHARSET "
+#endif /* MIME_BREAK_LONG_LINES */
+#ifdef NO_LOCALE
+				"+NO_LOCALE "
+#else
+				"-NO_LOCALE "
+#endif /* NO_LOCALE */
+				);
 				exit (EXIT_SUCCESS);
 				/* keep lint quiet: */
 				/* FALLTHROUGH */
-#ifndef INDEX_DAEMON
+
 			case 'w':	/* post article & exit */
 				post_article_and_exit = TRUE;
 				no_write = TRUE;
@@ -612,7 +753,6 @@ read_cmd_line_options (
 				check_any_unread = TRUE;
 				batch_mode = TRUE;
 				break;
-#endif /* !INDEX_DAEMON */
 
 			case 'h':
 			case '?':
@@ -634,9 +774,10 @@ read_cmd_line_options (
 			get_newsrcname(newsrc, uts.nodename);
 #else
 			char nodenamebuf[32];
-#	if defined(M_AMIGA)
+#	ifdef M_AMIGA
 			my_strncpy (nodenamebuf, get_val ("NodeName", "PROBLEM_WITH_NODE_NAME"), sizeof (nodenamebuf));
-#	else /* NeXT, Apollo */
+#	else
+			/* NeXT, Apollo */
 			(void) gethostname(nodenamebuf, sizeof(nodenamebuf));
 #	endif /* M_AMIGA */
 			get_newsrcname(newsrc, nodenamebuf);
@@ -645,12 +786,12 @@ read_cmd_line_options (
 	}
 
 	if (verbose && !batch_mode) {
-		wait_message(1, "-v only useful for batch mode operations\n");
+		wait_message(1, _("-v only useful for batch mode operations\n"));
 		verbose = FALSE;
 	}
 
 	if (read_saved_news && batch_mode) {
-		wait_message(1, "-R only useful without batch mode operations\n");
+		wait_message(1, _("-R only useful without batch mode operations\n"));
 		read_saved_news = FALSE;
 	}
 
@@ -685,158 +826,95 @@ static void
 usage (
 	char *theProgname)
 {
-#ifndef INDEX_DAEMON
-	error_message ("A Usenet reader.\n\nUsage: %s [options] [newsgroup[,...]]", theProgname);
+	error_message (_("A Usenet reader.\n\nUsage: %s [options] [newsgroup[,...]]"), theProgname);
 
 #	ifndef M_AMIGA
 #		ifdef HAVE_COLOR
-			error_message ("  -a       toggle color flag");
+			error_message (_("  -a       toggle color flag"));
 #		endif /* HAVE_COLOR */
 #		ifdef NNTP_ABLE
-			error_message ("  -A       force authentication on connect");
+			error_message (_("  -A       force authentication on connect"));
 #		endif /* NNTP_ABLE */
 #	else
-		error_message ("  -B       BBS mode. File operations limited to home directories.");
+		error_message (_("  -B       BBS mode. File operations limited to home directories."));
 #	endif /* !M_AMIGA */
 
-	error_message ("  -c       mark all news as read in subscribed newsgroups (batch mode)");
-	error_message ("  -d       don't show newsgroup descriptions");
+	error_message (_("  -c       mark all news as read in subscribed newsgroups (batch mode)"));
+	error_message (_("  -d       don't show newsgroup descriptions"));
 
 #	ifdef DEBUG
-		error_message ("  -D       debug mode 1=NNTP 2=ALL");
+		error_message (_("  -D       debug mode 1=NNTP 2=ALL"));
 #	endif /* DEBUG */
 
-	error_message ("  -f file  subscribed to newsgroups file [default=%s]", newsrc);
-	error_message ("  -G limit get only limit articles/group");
+	error_message (_("  -f file  subscribed to newsgroups file [default=%s]"), newsrc);
+	error_message (_("  -G limit get only limit articles/group"));
 
 #	ifndef M_AMIGA
 #		ifdef NNTP_ABLE
 			/* FIXME, default should be $NNTPSERVER if set ... */
-			error_message ("  -g serv  read news from NNTP server serv [default=%s]", NNTP_DEFAULT_SERVER);
+			error_message (_("  -g serv  read news from NNTP server serv [default=%s]"), NNTP_DEFAULT_SERVER);
 #		endif /* NNTP_ABLE */
 #	endif /* !M_AMIGA */
 
-	error_message ("  -h       this help message");
-	error_message ("  -H       help information about %s", theProgname);
+	error_message (_("  -h       this help message"));
+	error_message (_("  -H       help information about %s"), theProgname);
 
 #	ifndef NNTP_ONLY
-		error_message ("  -I dir   news index file directory [default=%s]", index_newsdir);
+		error_message (_("  -I dir   news index file directory [default=%s]"), index_newsdir);
 #	endif /* !NNTP_ONLY */
 
 #	ifdef NNTP_ABLE
-		error_message ("  -l       use only LISTGROUP instead of GROUP (-n) command");
-#	endif /* NNTP_ABLE */
-
-	error_message ("  -m dir   mailbox directory [default=%s]", tinrc.maildir);
-	error_message ("  -M user  mail new news to specified user (batch mode)");
-
-#	ifdef NNTP_ABLE
-		error_message ("  -n       only read subscribed .newsrc groups from NNTP server");
-#	endif /* NNTP_ABLE */
-
-	error_message ("  -N       mail new news to your posts");
-	error_message ("  -o       post all postponed articles and exit");
-
-#	ifdef NNTP_ABLE
-		error_message ("  -p port  use port as NNTP port [default=%d]", nntp_tcp_port);
-#	endif /* NNTP_ABLE */
-
-	error_message ("  -q       don't check for new newsgroups");
-
-#	ifdef NNTP_ABLE
-	error_message ("  -Q       quick start. Same as -nqd");
+		error_message (_("  -l       use only LISTGROUP instead of GROUP (-n) command"));
 #	else
-	error_message ("  -Q       quick start. Same as -qd");
+		error_message (_("  -l       TODO: document feature"));
+#	endif /* NNTP_ABLE */
+
+	error_message (_("  -m dir   mailbox directory [default=%s]"), tinrc.maildir);
+	error_message (_("  -M user  mail new news to specified user (batch mode)"));
+
+#	ifdef NNTP_ABLE
+		error_message (_("  -n       only read subscribed .newsrc groups from NNTP server"));
+#	else
+		error_message (_("  -n       TODO: document feature"));
+#	endif /* NNTP_ABLE */
+
+	error_message (_("  -N       mail new news to your posts (batch mode)"));
+	error_message (_("  -o       post all postponed articles and exit"));
+
+#	ifdef NNTP_ABLE
+		error_message (_("  -p port  use port as NNTP port [default=%d]"), nntp_tcp_port);
+#	endif /* NNTP_ABLE */
+
+	error_message (_("  -q       don't check for new newsgroups"));
+
+#	ifdef NNTP_ABLE
+	error_message (_("  -Q       quick start. Same as -nqd"));
+#	else
+	error_message (_("  -Q       quick start. Same as -qd"));
 #	endif /* NNTP_ABLE */
 
 #	ifdef NNTP_ABLE
 		if (!read_news_via_nntp)
-			error_message ("  -r       read news remotely from default NNTP server");
+			error_message (_("  -r       read news remotely from default NNTP server"));
 #	endif /* NNTP_ABLE */
 
-	error_message ("  -R       read news saved by -S option");
-	error_message ("  -s dir   save news directory [default=%s]", tinrc.savedir);
-	error_message ("  -S       save new news for later reading (batch mode)");
+	error_message (_("  -R       read news saved by -S option"));
+	error_message (_("  -s dir   save news directory [default=%s]"), tinrc.savedir);
+	error_message (_("  -S       save new news for later reading (batch mode)"));
 
 #	ifndef NNTP_ONLY
-		error_message ("  -u       update index files (batch mode)");
-		error_message ("  -U       update index files in the background while reading news");
+		error_message (_("  -u       update index files (batch mode)"));
+		error_message (_("  -U       update index files in the background while reading news"));
 #	endif /* !NNTP_ONLY */
 
-	error_message ("  -v       verbose output for batch mode options");
-	error_message ("  -V       print version & date information");
-	error_message ("  -w       post an article and exit");
-	error_message ("  -X       don't save any files on quit");
-	error_message ("  -z       start if any unread news");
-	error_message ("  -Z       return status indicating if any unread news (batch mode)");
+	error_message (_("  -v       verbose output for batch mode options"));
+	error_message (_("  -V       print version & date information"));
+	error_message (_("  -w       post an article and exit"));
+	error_message (_("  -X       don't save any files on quit"));
+	error_message (_("  -z       start if any unread news"));
+	error_message (_("  -Z       return status indicating if any unread news (batch mode)"));
 
-#else /* INDEX_DAEMON */
-
-	error_message ("Index file daemon.\n\nUsage: %s [options] [newsgroups]", theProgname);
-
-	error_message ("  -d       delete index file before indexing articles");
-
-#	ifdef DEBUG
-	error_message ("  -D       debug mode 1=NNTP 2=ALL");
-#	endif /* DEBUG */
-
-	error_message ("  -f file  active newsgroups file [default=%s]", newsrc);
-	error_message ("  -h       this help message");
-	error_message ("  -I dir   news index file directory [default=%s]", index_newsdir);
-	error_message ("  -P       purge any expired articles from index files");
-	error_message ("  -v       verbose output for batch mode options");
-	error_message ("  -V       print version & date information");
-#endif /* INDEX_DAEMON */
-
-	error_message ("\nMail bug reports/comments to %s", BUG_REPORT_ADDRESS);
-}
-
-
-/*
- * check/start if any new/unread articles
- */
-static int
-check_for_any_new_news (
-	t_bool CheckAnyUnread,
-	t_bool StartAnyUnread)
-{
-	int i = 0;
-
-	if (CheckAnyUnread) {
-		i = check_start_save_any_news (CHECK_ANY_NEWS);
-		cursoron();
-		Raw (FALSE);
-		exit (i);
-	}
-
-	if (StartAnyUnread) {
-		batch_mode = TRUE;			/* Suppress some unwanted on-screen garbage */
-		if ((i = check_start_save_any_news (START_ANY_NEWS)) == -1)
-			giveup();				/* No new/unread news so exit */
-		batch_mode = FALSE;
-	}
-
-	return (i);
-}
-
-
-/*
- * mail any new articles to specified user
- * or
- * save any new articles to savedir structure for later reading
- */
-static void
-save_or_mail_new_news (
-	void)
-{
-	if (mail_news || save_news) {
-		t_bool i = catchup;
-		catchup = FALSE;	/* set catchup to FALSE */
-		do_update ();
-		catchup = i;		/* set catchup to previous value */
-		check_start_save_any_news (mail_news ? MAIL_ANY_NEWS : SAVE_ANY_NEWS);
-		tin_done (EXIT_SUCCESS);
-	}
+	error_message (_("\nMail bug reports/comments to %s"), bug_addr);
 }
 
 
@@ -847,83 +925,23 @@ static void
 update_index_files (
 	void)
 {
-	if (batch_mode || update_fork) {
-		if (!catchup && (read_news_via_nntp && xover_supported)) {
-			error_message (txt_batch_update_unavail, tin_progname);
-			tin_done (EXIT_FAILURE);
-		}
-
-		cCOLS = 132;				/* set because curses has not started */
-#ifdef HAVE_FORK
-		if (update_fork) {
-			catchup = FALSE;		/* turn off msgs when running forked */
-			verbose = FALSE;
-			switch ((int) fork ()) {		/* fork child to update indexes in background */
-				case -1:			/* error forking */
-					perror_message (txt_batch_update_failed);
-					break;
-				case 0:				/* child process */
-					create_index_lock_file (lock_file);
-					process_id = getpid ();
-#	if defined(BSD) /* FIXME: check for setsid/setpgid/... and remove OS depending ifdefs */
-#		if defined(__FreeBSD__) || defined(__NetBSD__)
-					setsid();
-#		else
-#			ifdef __osf__
-					setpgid (0, 0);
-#			else
-					setpgrp (0, process_id);	/* reset process group leader to this process */
-#			endif /* __osf__ */
-#			ifdef TIOCNOTTY
-					{
-						int fd;
-
-						if ((fd = open ("/dev/tty", O_RDWR)) >= 0) {
-							ioctl (fd, TIOCNOTTY, (char *) NULL);
-							close (fd);
-						}
-					}
-#			endif /* TIOCNOTTY */
-#		endif /* __FreeBSD__ || __NetBSD__ */
-#	else
-#		ifdef HAVE_SETPGRP
-#				ifdef SETPGRP_VOID
-						setpgrp ();
-#				else
-						setpgrp (0, process_id);
-#				endif /* SETPGRP_VOID */
-					signal (SIGHUP, SIG_IGN);	/* make immune from process group leader death */
-#		endif /* HAVE_SETPGRP */
-#	endif /* BSD */
-					signal (SIGQUIT, SIG_IGN);	/* stop indexing being interrupted */
-
-					if (nntp_open () != 0)				/* connect server if we are using nntp */
-						tin_done (EXIT_SUCCESS);
-
-					tinrc.thread_articles = THREAD_NONE;	/* stop threading to run faster */
-					do_update ();
-					tin_done (EXIT_SUCCESS);
-					break;
-				default:						/* parent process*/
-					break;
-			}
-			batch_mode = FALSE;
-		} else
-#endif /* HAVE_FORK */
-		{
-			create_index_lock_file (lock_file);
-			tinrc.thread_articles = THREAD_NONE;	/* stop threading to run faster */
-			do_update ();
-			tin_done (EXIT_SUCCESS);
-		}
+	if (!catchup && (read_news_via_nntp && xover_supported)) {
+		error_message (_(txt_batch_update_unavail), tin_progname);
+		tin_done (EXIT_FAILURE);
 	}
+
+	cCOLS = 132;							/* set because curses has not started */
+	batch_mode = TRUE;					/* -u handling... */
+	create_index_lock_file (lock_file);
+	tinrc.thread_articles = THREAD_NONE;	/* stop threading to run faster */
+	do_update (catchup);
+	tin_done (EXIT_SUCCESS);
 }
 
 
 /*
  * display page of general info. for first time user.
  */
-#ifndef INDEX_DAEMON
 static void
 show_intro_page (
 	void)
@@ -937,17 +955,16 @@ show_intro_page (
 		my_printf("\n");
 	}
 
-	sprintf(buf, txt_intro_page, BUG_REPORT_ADDRESS);
+	sprintf(buf, _(txt_intro_page), bug_addr);
 
 	my_fputs (buf, stdout);
 	my_flush();
 
 	if (!cmd_line) {
 		Raw (TRUE);
-		continue_prompt ();
+		prompt_continue ();
 	}
 }
-#endif /* !INDEX_DAEMON */
 
 
 /*
@@ -963,10 +980,10 @@ read_cmd_line_groups (
 	register int i;
 
 	if (num_cmdargs < max_cmdargs) {
-		group_top = skip_newgroups();		/* Reposition after any newgroups */
+		selmenu.max = skip_newgroups();		/* Reposition after any newgroups */
 
 		for (num = num_cmdargs; num < max_cmdargs; num++) {
-			wait_message (0, txt_matching_cmd_line_groups, cmdargs[num]);
+			wait_message (0, _(txt_matching_cmd_line_groups), cmdargs[num]);
 
 			for (i = 0; i < num_active; i++) {
 				if (match_group_list (active[i].name, cmdargs[num])) {
@@ -981,7 +998,8 @@ read_cmd_line_groups (
 }
 
 void
-giveup(void)
+giveup (
+	void)
 {
 	static int nested;
 
