@@ -5,13 +5,39 @@
  *  Created   : 1991-04-01
  *  Updated   : 1997-12-31
  *  Notes     :
- *  Copyright : (c) Copyright 1991-99 by Iain Lea
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Iain Lea.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
@@ -55,6 +81,7 @@ prompt_num (
  *  prompt_string
  *  get a string from the user
  *  Return TRUE if a valid string was typed, FALSE otherwise
+ *  TODO no bounds checking on buf size, tin_getline() defaults to 1024
  */
 t_bool
 prompt_string (
@@ -131,6 +158,7 @@ prompt_menu_string (
  * prompt_yn
  * prompt user for 'y'es or 'n'o decision. "prompt" will be displayed in line
  * "line" giving the default answer "default_answer".
+ * TODO 'line' is constant - can we remove it ?
  * The function returns 1 if the user decided "yes", -1 if the user wanted
  * to escape, or 0 for any other key or decision.
  */
@@ -140,24 +168,32 @@ prompt_yn (
 	const char *prompt,
 	t_bool default_answer)
 {
-	int ch, prompt_ch;
+	char *keyprompt;
+	char keyno[MAXKEYLEN], keyyes[MAXKEYLEN];
+	int ch = 'y', prompt_ch = 'y';
+	size_t maxlen;
 	t_bool yn_loop = TRUE;
 
-/*	fflush(stdin);*/		/* Prevent finger trouble from making important decisions */
+/*	fflush(stdin); */		/* Prevent finger trouble from making important decisions */
+
+	(void) printascii (keyyes, (default_answer ? toupper(map_to_local(iKeyPromptYes, &menukeymap.prompt_yn)) : map_to_local(iKeyPromptYes, &menukeymap.prompt_yn)));
+	(void) printascii (keyno, (!default_answer ? toupper(map_to_local(iKeyPromptNo, &menukeymap.prompt_yn)) : map_to_local(iKeyPromptNo, &menukeymap.prompt_yn)));
+	maxlen = MAX(strlen (keyyes), strlen (keyno));
 
 	while (yn_loop) {
-		prompt_ch = (default_answer ? iKeyPromptYes : iKeyPromptNo);
+		prompt_ch = map_to_local ((default_answer ? iKeyPromptYes : iKeyPromptNo), &menukeymap.prompt_yn);
+		keyprompt = (default_answer ? keyyes : keyno);
 
 		if (!cmd_line) {
 			MoveCursor (line, 0);
 			CleartoEOLN ();
 		}
-		my_printf ("%s%c", prompt, prompt_ch);
+		my_printf ("%s (%s/%s) %-*s", prompt, keyyes, keyno, (int)maxlen, keyprompt);
 		if (!cmd_line)
 			cursoron ();
 		my_flush ();
 		if (!cmd_line)
-			MoveCursor (line, (int) strlen (prompt));
+			MoveCursor (line, (int) strlen (prompt) + strlen(keyyes) + strlen(keyno) + 5);
 
 		if (((ch = (char) ReadCh()) == '\n') || (ch == '\r'))
 			ch = prompt_ch;
@@ -179,7 +215,7 @@ prompt_yn (
 						break;
 
 					case KEYMAP_LEFT:
-						ch = ESC;
+						ch = iKeyAbort;
 						break;
 
 					case KEYMAP_RIGHT:
@@ -201,13 +237,13 @@ prompt_yn (
 			clear_message ();
 		else {
 			MoveCursor (line, (int) strlen (prompt));
-			my_fputc (((ch == ESC) ? prompt_ch : ch), stdout);
+			my_fputc (((ch == iKeyAbort) ? prompt_ch : ch), stdout);
 		}
 		cursoroff ();
 		my_flush ();
 	}
 
-	return (tolower ((unsigned char)ch) == tolower ((unsigned char)iKeyPromptYes)) ? 1 : (ch == ESC) ? -1 : 0;
+	return (tolower ((unsigned char) map_to_default (ch, &menukeymap.prompt_yn)) == tolower ((unsigned char)iKeyPromptYes)) ? 1 : (ch == iKeyAbort) ? -1 : 0;
 }
 
 
@@ -231,7 +267,7 @@ prompt_list (
 {
 	int ch, var_orig;
 	int i;
-	int adjust = (strcasecmp(list[0], txt_default) == 0);
+	int adjust = (strcasecmp(list[0], _(txt_default)) == 0);
 	size_t width = 0;
 
 	var += adjust;
@@ -441,7 +477,7 @@ prompt_msgid (
 {
 	char buf[LEN];
 
-	if (prompt_string(txt_enter_message_id, buf+1, HIST_MESSAGE_ID) && buf[1]) {
+	if (prompt_string(_(txt_enter_message_id), buf+1, HIST_MESSAGE_ID) && buf[1]) {
 		char *ptr = buf+1;
 		struct t_msgid *msgid;
 
@@ -455,7 +491,7 @@ prompt_msgid (
 		}
 
 		if ((msgid = find_msgid(ptr)) == NULL) {
-			info_message(txt_art_unavailable);
+			info_message(_(txt_art_unavailable));
 			return ART_UNAVAILABLE;
 		}
 
@@ -463,7 +499,7 @@ prompt_msgid (
 		 * Is it expired or otherwise not on the spool ?
 		 */
 		if (msgid->article == ART_UNAVAILABLE) {
-			info_message(txt_art_unavailable);
+			info_message(_(txt_art_unavailable));
 			return ART_UNAVAILABLE;
 		}
 
@@ -472,7 +508,7 @@ prompt_msgid (
 		 * no way to display it
 		 */
 		if (which_thread(msgid->article) == -1) {
-			info_message (txt_no_last_message);
+			info_message (_(txt_no_last_message));
 			return ART_UNAVAILABLE;
 		}
 
@@ -513,7 +549,7 @@ sized_message (
 int
 prompt_slk_response (
 	int ch_default,
-	const char *responses,
+	const t_menukeys /* char */ *responses,
 	const char *fmt, ...)
 {
 	va_list ap;
@@ -521,9 +557,10 @@ prompt_slk_response (
 	char buf[LEN];
 
 	va_start (ap, fmt);
-	vsprintf(buf, fmt, ap);	/* We need to do this, else wait_message() will clobber us */
+	vsnprintf (buf, sizeof(buf) - 1, fmt, ap);	/* We need to do this, else wait_message() will clobber us */
 	va_end (ap);
 
+	ch_default = map_to_local (ch_default, responses);
 	do {
 		wait_message (0, "%s%c", buf, ch_default);
 
@@ -533,9 +570,10 @@ prompt_slk_response (
 		if ((ch = ReadCh ()) == '\r' || ch == '\n')
 			ch = ch_default;
 
-	} while (!strchr (responses, ch));
+	} while (!strchr (responses->localkeys, ch));
 
-	return(ch);
+	clear_message ();
+	return map_to_default (ch, responses);
 }
 
 
@@ -546,7 +584,7 @@ prompt_slk_response (
  * ii) CTRL, SHIFT etc don't work
  */
 void
-continue_prompt (
+prompt_continue (
 	void)
 {
 	int ch;
@@ -554,7 +592,7 @@ continue_prompt (
 #ifdef USE_CURSES
 	cmd_line = TRUE;
 #endif /* USE_CURSES */
-	info_message (txt_return_key);
+	info_message (_(txt_return_key));
 	ch = ReadCh ();
 
 #ifndef WIN32
