@@ -4,13 +4,39 @@
  *  Author    : Jason Faultless <jason@radar.tele2.co.uk>
  *  Created   : 1997-04-10
  *  Updated   : 1998-07-04
- *  Copyright : (c) 1997-99 by Jason Faultless
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1997-2001 Jason Faultless <jason@radar.tele2.co.uk>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Jason Faultless.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
@@ -45,18 +71,19 @@ static char * tin_read (char *buffer, size_t len, FILE *fp, t_bool header);
  *         FALSE otherwise
  */
 static t_bool
-wait_for_input (void /*FILE *fd*/)
+wait_for_input (
+	void /*FILE *fd*/)
 {
 #	ifndef HAVE_SELECT
 #		ifdef VMS
 	int ch = ReadChNowait ();
 
 	if (ch == 'q' || ch == 'z' || ch == ESC) {
-		if (prompt_yn (cLINES, txt_read_abort, FALSE) == 1)
+		if (prompt_yn (cLINES, _(txt_read_abort), FALSE) == 1)
 			return TRUE;
 	}
 	if (ch == 'Q') {
-		if (prompt_yn (cLINES, txt_read_exit, FALSE) == 1)
+		if (prompt_yn (cLINES, _(txt_read_exit), FALSE) == 1)
 			tin_done (EXIT_SUCCESS);
 	}
 #		endif /* VMS */
@@ -104,12 +131,12 @@ wait_for_input (void /*FILE *fd*/)
 					return FALSE;
 
 				if (ch == 'q' || ch == 'z' || ch == ESC) {
-					if (prompt_yn (cLINES, txt_read_abort, FALSE) == 1)
+					if (prompt_yn (cLINES, _(txt_read_abort), FALSE) == 1)
 						return TRUE;
 				}
 
 				if (ch == 'Q') {
-					if (prompt_yn (cLINES, txt_read_exit, FALSE) == 1)
+					if (prompt_yn (cLINES, _(txt_read_exit), FALSE) == 1)
 						tin_done (EXIT_SUCCESS);
 				}
 
@@ -171,7 +198,7 @@ tin_read (
 
 #ifdef NNTP_ABLE
 	if (wait_for_input(/*fp*/)) {			/* Check if okay to read */
-		info_message("Aborting read, please wait...");
+		info_message(_("Aborting read, please wait..."));
 		drain_buffer(fp);
 		clear_message();
 		tin_errno = TIN_ABORT;
@@ -313,7 +340,7 @@ tin_fgets (
 		return ptr;
 
 	if (tin_errno != 0) {
-		DEBUG_IO((stderr, "Aborted read\n"));
+		DEBUG_IO((stderr, _("Aborted read\n")));
 		return(NULL);
 	}
 
@@ -369,7 +396,7 @@ drain_buffer (
 	if (fp != FAKE_NNTP_FP)
 		return;
 
-DEBUG_IO((stderr, "Draining\n"));
+DEBUG_IO((stderr, _("Draining\n")));
 	while (tin_fgets(fp, FALSE) != (char *) 0) {
 		if (++i % MODULO_COUNT_NUM == 0)
 			spin_cursor();
