@@ -7,13 +7,39 @@
  *  Notes     : Cacheing of message ids / References based threading
  *  Credits   : Richard Hodson <richard@macgyver.tele2.co.uk>
  *              hash_msgid, free_msgid
- *  Copyright : (c) 1996-99 by Jason Faultless
- *              You may  freely  copy or  redistribute  this software,
- *              so  long as there is no profit made from its use, sale
- *              trade or  reproduction.  You may not change this copy-
- *              right notice, and it must be included in any copy made
+ *
+ * Copyright (c) 1996-2001 Jason Faultless <jason@radar.tele2.co.uk>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Jason Faultless.
+ * 4. The name of the author may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
+ * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
+ * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
@@ -35,10 +61,10 @@ FILE *dbgfd;
  * local prototypes
  */
 static char * _get_references (struct t_msgid *refptr, int depth);
-static struct t_msgid * add_msgid (int key, char *msgid, struct t_msgid *newparent);
-static struct t_msgid * find_next (struct t_msgid *ptr);
-static struct t_msgid * parse_references (char *r);
-static unsigned int hash_msgid (char *key);
+static struct t_msgid *add_msgid (int key, const char *msgid, struct t_msgid *newparent);
+static struct t_msgid *find_next (struct t_msgid *ptr);
+static struct t_msgid *parse_references (char *r);
+static unsigned int hash_msgid (const char *key);
 static void add_to_parent (struct t_msgid *ptr);
 static void build_thread (struct t_msgid *ptr);
 #ifdef DEBUG_REFS
@@ -92,7 +118,7 @@ static struct t_msgid *msgids[MSGID_HASH_SIZE] = {0};
  */
 static unsigned int
 hash_msgid (
-	char *key)
+	const char *key)
 {
 	unsigned int hash = 0;
 
@@ -178,7 +204,7 @@ add_to_parent (
 static struct t_msgid *
 add_msgid (
 	int key,
-	char *msgid,
+	const char *msgid,
 	struct t_msgid *newparent)
 {
 	struct t_msgid *ptr;
@@ -192,7 +218,7 @@ add_msgid (
 
 	h = hash_msgid(msgid+1);				/* Don't hash the initial '<' */
 
-	DEBUG_PRINT((dbgfd, "---------------- Add %s %s with parent %s\n", (key==MSGID_REF)?"MSG":"REF", msgid, (newparent == NULL) ? "unchanged" : newparent->txt));
+	DEBUG_PRINT((dbgfd, "---------------- Add %s %s with parent %s\n", (key == MSGID_REF) ? "MSG" : "REF", msgid, (newparent == NULL) ? _("unchanged") : newparent->txt));
 
 	/*
 	 * Look for this message id in the cache.
@@ -231,7 +257,7 @@ add_msgid (
 			 */
 			for (ptr = newparent; ptr != NULL; ptr = ptr->parent) {
 				if (ptr == i) {
-					DEBUG_PRINT((dbgfd, "Avoiding circular reference! (%s)\n", (key==MSGID_REF) ? "MSG" : "REF"));
+					DEBUG_PRINT((dbgfd, "Avoiding circular reference! (%s)\n", (key == MSGID_REF) ? "MSG" : "REF"));
 					return(i);
 				}
 			}
@@ -239,7 +265,7 @@ add_msgid (
 			i->parent = newparent;
 			add_to_parent(i);
 
-			DEBUG_PRINT((dbgfd, "set: %s -> %s\n", i->txt, newparent ? newparent->txt : "None"));
+			DEBUG_PRINT((dbgfd, "set: %s -> %s\n", i->txt, newparent ? newparent->txt : _("None")));
 			return(i);
 		}
 
@@ -252,7 +278,7 @@ add_msgid (
 		 */
 		if (i->parent != newparent) {
 			DEBUG_PRINT((dbgfd, "Warning: (%s) Ignoring %s -> %s (already %s)\n",
-				(key==MSGID_REF) ? "MSG" : "REF", i->txt,
+				(key == MSGID_REF) ? "MSG" : "REF", i->txt,
 				newparent ? newparent->txt : "None", i->parent->txt));
 
 			return(i);
@@ -272,7 +298,7 @@ add_msgid (
 	strcpy(ptr->txt, msgid);
 	ptr->parent = newparent;
 	ptr->child = ptr->sibling = NULL;
-	ptr->article = (key == MSGID_REF ? top : ART_UNAVAILABLE);
+	ptr->article = (key == MSGID_REF ? top_art : ART_UNAVAILABLE);
 
 	add_to_parent(ptr);
 
@@ -291,7 +317,7 @@ add_msgid (
  */
 struct t_msgid *
 find_msgid (
-	char *msgid)
+	const char *msgid)
 {
 	unsigned int h;
 	struct t_msgid *i;
@@ -450,7 +476,7 @@ dump_msgids (
 
 	my_fprintf(stderr, "Dumping...\n");
 
-	for (i=0; i<MSGID_HASH_SIZE; i++) {
+	for (i = 0; i<MSGID_HASH_SIZE; i++) {
 
 		if (msgids[i] != NULL) {
 
@@ -482,7 +508,7 @@ dump_msgids (
  * . The References: field should not be truncated, though in practice
  *   this will happen, often in badly broken ways.
  *
- * This is simplistic, so check out RFC1036 & son of RFC1036 for full
+ * This is simplistic, so check out RFC 1036 & son of RFC 1036 for full
  * details from the posting point of view.
  *
  * We attempt to maintain 3 pointers in each message-id to handle threading
@@ -548,7 +574,7 @@ dump_thread (
 	if (msgid->article >= 0)
 		sprintf(ptr+len, "%-*.*s   %-17.17s", i, i, arts[msgid->article].subject, (arts[msgid->article].name) ? arts[msgid->article].name : arts[msgid->article].from);
 	else
-		sprintf(ptr+len, "%-*.*s", i, i, "[- Unavailable -]");
+		sprintf(ptr+len, "%-*.*s", i, i, _("[- Unavailable -]"));
 
 	fprintf(fp, "%s\n", ptr);
 
@@ -591,7 +617,7 @@ dump_msgid_threads (
 
 	fprintf(dbgfd, "Dump started.\n\n");
 
-	for (i=0; i<MSGID_HASH_SIZE; i++) {
+	for (i = 0; i<MSGID_HASH_SIZE; i++) {
 		if (msgids[i] != NULL) {
 
 			for (ptr = msgids[i]; ptr != NULL; ptr = ptr->next) {
@@ -736,9 +762,9 @@ thread_by_reference (
 	/*
 	 * Build threads starting from root msgids (ie without parent)
 	 */
-	for (i=0; i<MSGID_HASH_SIZE; i++) {
+	for (i = 0; i<MSGID_HASH_SIZE; i++) {
 		if (msgids[i] != NULL) {
-			for (ptr = msgids[i]; ptr!=NULL; ptr = ptr->next) {
+			for (ptr = msgids[i]; ptr != NULL; ptr = ptr->next) {
 				if (ptr->parent == NULL)
 					build_thread(ptr);
 			}
@@ -749,7 +775,7 @@ thread_by_reference (
 	fprintf(dbgfd, "Full dump of threading info...\n");
 	fprintf(dbgfd, "%3s %3s %3s %3s : %3s %3s\n", "#", "Par", "Sib", "Chd", "In", "Thd");
 
-	for (i = 0; i < top; i++) {
+	for (i = 0; i < top_art; i++) {
 		fprintf(dbgfd, "%3d %3d %3d %3d : %3d %3d : %.50s %s\n", i,
 			(arts[i].refptr->parent) ? arts[i].refptr->parent->article : -2,
 			(arts[i].refptr->sibling) ? arts[i].refptr->sibling->article : -2,
@@ -785,7 +811,7 @@ collate_subjects (
 	 * Run through the root messages of each thread. We have to traverse
 	 * using arts[] and not msgids[] to preserve the sorting.
 	 */
-	for (i = 0; i < top; i++) {
+	for (i = 0; i < top_art; i++) {
 
 		/*
 		 * Ignore already threaded and expired arts
@@ -836,7 +862,7 @@ collate_subjects (
  * 1) Sort the article base. This will ensure that articles and their
  *    siblings are inserted in the correct order.
  * 2) Add each Message-ID header and its direct reference ('reliable info')
- *    to the cache. Son of RFC1036 mandates that if References headers must
+ *    to the cache. Son of RFC 1036 mandates that if References headers must
  *    be trimmed, then at least the (1st three and) last reference should be
  *    maintained.
  * 3) Add rest of References header to the cache. This information is less
@@ -852,7 +878,6 @@ build_references (
 	struct t_article *art;
 	char *s;
 
-#ifndef INDEX_DAEMON
 	/*
 	 * The articles are currently unsorted, and are as they were put by setup_hard_base()
 	 */
@@ -864,7 +889,6 @@ build_references (
 		               group->attribute->sort_art_type == SORT_BY_FROM_ASCEND ||
 		               group->attribute->sort_art_type == SORT_BY_DATE_ASCEND ||
 		               group->attribute->sort_art_type == SORT_BY_SCORE_ASCEND);
-#endif /* !INDEX_DAEMON */
 
 #ifdef DEBUG_REFS
 	dbgfd = fopen("Refs.dump", "w");
@@ -876,7 +900,7 @@ build_references (
 	 * Add the Message-ID headers to the cache, using the last Reference
 	 * as the parent
 	 */
-	for (i = 0; i < top; i++) {
+	for (i = 0; i < top_art; i++) {
 		art = &arts[i];
 
 		if (art->refs) {
@@ -914,7 +938,7 @@ build_references (
 	/*
 	 * Add the References data to the cache
 	 */
-	for (i = 0; i < top; i++) {
+	for (i = 0; i < top_art; i++) {
 		struct t_msgid *refs;
 
 		if (!arts[i].refs)						/* No refs - skip */
